@@ -56,6 +56,7 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <stdio.h>
+#include <string>
 
 namespace ssh {
 
@@ -193,6 +194,43 @@ public:
     ssh_throw(ret);
     return ret;
   }
+
+  /** @brief Authenticate through the "keyboard-interactive" method.
+   * @param[in] The username to authenticate. You can specify NULL if ssh_option_set_username() has been used. You cannot try two different logins in a row.
+   * @param[in] Undocumented. Set it to NULL.
+   * @throws SshException on error
+   * @returns SSH_AUTH_SUCCESS, SSH_AUTH_PARTIAL, SSH_AUTH_DENIED, SSH_AUTH_ERROR, SSH_AUTH_INFO, SSH_AUTH_AGAIN
+   * @see ssh_userauth_kbdint
+   */
+  int userauthKbdint(const char* username, const char* submethods){
+    int ret=ssh_userauth_kbdint(c_session,NULL,NULL);
+    ssh_throw(ret);
+    return ret;
+  }
+
+  /** @brief Get the number of prompts (questions) the server has given.
+   * @returns The number of prompts.
+   * @see ssh_userauth_kbdint_getnprompts
+   */
+  int userauthKbdintGetNPrompts(){
+    return ssh_userauth_kbdint_getnprompts(c_session);
+  }
+
+  /** @brief Set the answer for a question from a message block..
+   * @param[in] index The number of the ith prompt.
+   * @param[in] The answer to give to the server. The answer MUST be encoded UTF-8. It is up to the server how to interpret the value and validate it. However, if you read the answer in some other encoding, you MUST convert it to UTF-8.
+   * @throws SshException on error
+   * @returns 0 on success, < 0 on error
+   * @see ssh_userauth_kbdint_setanswer
+   */
+  int userauthKbdintSetAnswer(unsigned int i, const char* answer){
+    int ret=ssh_userauth_kbdint_setanswer(c_session, i, answer);
+    ssh_throw(ret);
+    return ret;
+  }
+
+
+
   /** @brief Authenticates using the password method.
    * @param[in] password password to use for authentication
    * @throws SshException on error
@@ -227,8 +265,7 @@ public:
     ssh_throw(ret);
     return ret;
   }
-  int userauthPrivatekeyFile(const char *filename,
-      const char *passphrase);
+
   /** @brief Returns the available authentication methods from the server
    * @throws SshException on error
    * @returns Bitfield of available methods.
@@ -280,8 +317,12 @@ public:
    */
   std::string getIssueBanner(){
     char *banner=ssh_get_issue_banner(c_session);
-    std::string ret= std::string(banner);
-    ::free(banner);
+    std::string ret;
+    if (banner)
+    {
+      ret= std::string(banner);
+      ::free(banner);
+    }
     return ret;
   }
   /** @brief returns the OpenSSH version (server) if possible
@@ -302,12 +343,12 @@ public:
    * @throws SshException on error
    * @returns Integer value depending on the knowledge of the
    * server key
-   * @see ssh_is_server_known
+   * @see ssh_session_update_known_hosts
    */
   int isServerKnown(){
-    int ret=ssh_is_server_known(c_session);
-    ssh_throw(ret);
-    return ret;
+    int state = ssh_session_is_known_server(c_session);
+    ssh_throw(state);
+    return state;
   }
   void log(int priority, const char *format, ...){
     char buffer[1024];
@@ -361,27 +402,30 @@ public:
    * @see ssh_channel_forward_accept
    * @see Session::listenForward
    */
-  Channel *acceptForward(int timeout_ms);
-  /* acceptForward is implemented later in this file */
+  inline Channel *acceptForward(int timeout_ms);
+  /* implemented outside the class due Channel references */
 
   void_throwable cancelForward(const char *address, int port){
-    int err=ssh_forward_cancel(c_session, address, port);
+    int err=ssh_channel_cancel_forward(c_session, address, port);
     ssh_throw(err);
     return_throwable;
   }
 
   void_throwable listenForward(const char *address, int port,
       int &boundport){
-    int err=ssh_forward_listen(c_session, address, port, &boundport);
+    int err=ssh_channel_listen_forward(c_session, address, port, &boundport);
     ssh_throw(err);
     return_throwable;
   }
 
-private:
-  ssh_session c_session;
   ssh_session getCSession(){
     return c_session;
   }
+
+protected:
+  ssh_session c_session;
+
+private:
   /* No copy constructor, no = operator */
   Session(const Session &);
   Session& operator=(const Session &);
@@ -480,12 +524,30 @@ public:
     ssh_throw(err);
     return err;
   }
-  int read(void *dest, size_t count, bool is_stderr=false){
+  int read(void *dest, size_t count){
     int err;
     /* handle int overflow */
     if(count > 0x7fffffff)
       count = 0x7fffffff;
-    err=ssh_channel_read(channel,dest,count,is_stderr);
+    err=ssh_channel_read_timeout(channel,dest,count,false,-1);
+    ssh_throw(err);
+    return err;
+  }
+  int read(void *dest, size_t count, int timeout){
+    int err;
+    /* handle int overflow */
+    if(count > 0x7fffffff)
+      count = 0x7fffffff;
+    err=ssh_channel_read_timeout(channel,dest,count,false,timeout);
+    ssh_throw(err);
+    return err;
+  }
+  int read(void *dest, size_t count, bool is_stderr=false, int timeout=-1){
+    int err;
+    /* handle int overflow */
+    if(count > 0x7fffffff)
+      count = 0x7fffffff;
+    err=ssh_channel_read_timeout(channel,dest,count,is_stderr,timeout);
     ssh_throw(err);
     return err;
   }
@@ -565,26 +627,33 @@ public:
     ssh_throw(ret);
     return ret;
   }
-private:
+
   ssh_session getCSession(){
     return session->getCSession();
   }
+
+  ssh_channel getCChannel() {
+    return channel;
+  }
+
+protected:
+  Session *session;
+  ssh_channel channel;
+
+private:
   Channel (Session &session, ssh_channel c_channel){
     this->channel=c_channel;
     this->session=&session;
   }
-  Session *session;
-  ssh_channel channel;
   /* No copy and no = operator */
   Channel(const Channel &);
   Channel &operator=(const Channel &);
 };
 
 
-/* This code cannot be put inline due to references to Channel */
-Channel *Session::acceptForward(int timeout_ms){
-    ssh_channel forward = ssh_forward_accept(c_session,
-        timeout_ms);
+inline Channel *Session::acceptForward(int timeout_ms){
+    ssh_channel forward =
+        ssh_channel_accept_forward(c_session, timeout_ms, NULL);
     ssh_throw_null(c_session,forward);
     Channel *newchan = new Channel(*this,forward);
     return newchan;
