@@ -43,24 +43,7 @@ enum ssh_packet_state_e {
   PACKET_STATE_PROCESSING
 };
 
-enum ssh_packet_filter_result_e {
-    SSH_PACKET_UNKNOWN,
-    SSH_PACKET_ALLOWED,
-    SSH_PACKET_DENIED
-};
-
-int packet_send(ssh_session session);
-
-#ifdef WITH_SSH1
-int packet_send1(ssh_session session) ;
-void ssh_packet_set_default_callbacks1(ssh_session session);
-
-SSH_PACKET_CALLBACK(ssh_packet_disconnect1);
-SSH_PACKET_CALLBACK(ssh_packet_smsg_success1);
-SSH_PACKET_CALLBACK(ssh_packet_smsg_failure1);
-int ssh_packet_socket_callback1(const void *data, size_t receivedlen, void *user);
-
-#endif
+int ssh_packet_send(ssh_session session);
 
 SSH_PACKET_CALLBACK(ssh_packet_unimplemented);
 SSH_PACKET_CALLBACK(ssh_packet_disconnect_callback);
@@ -84,12 +67,13 @@ void ssh_packet_set_default_callbacks(ssh_session session);
 void ssh_packet_process(ssh_session session, uint8_t type);
 
 /* PACKET CRYPT */
-uint32_t packet_decrypt_len(ssh_session session, char *crypted);
-int packet_decrypt(ssh_session session, void *packet, unsigned int len);
-unsigned char *packet_encrypt(ssh_session session,
-                              void *packet,
-                              unsigned int len);
-int packet_hmac_verify(ssh_session session,ssh_buffer buffer,
-                       unsigned char *mac, enum ssh_hmac_e type);
+uint32_t ssh_packet_decrypt_len(ssh_session session, uint8_t *destination, uint8_t *source);
+int ssh_packet_decrypt(ssh_session session, uint8_t *destination, uint8_t *source,
+        size_t start, size_t encrypted_size);
+unsigned char *ssh_packet_encrypt(ssh_session session,
+                                  void *packet,
+                                  unsigned int len);
+int ssh_packet_hmac_verify(ssh_session session,ssh_buffer buffer,
+                           unsigned char *mac, enum ssh_hmac_e type);
 
 #endif /* PACKET_H_ */
