@@ -21,9 +21,9 @@
  * MA 02111-1307, USA.
  */
 
+#include "config.h"
+
 #include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 
 #ifndef _WIN32
 #include <netinet/in.h>
@@ -171,16 +171,13 @@ static int callback_receive_banner(const void *data, size_t len, void *user)
  */
 int ssh_send_banner(ssh_session session, int server)
 {
-    const char *banner = NULL;
-    const char *terminator = NULL;
+    const char *banner = CLIENT_BANNER_SSH2;
+    const char *terminator = "\r\n";
     /* The maximum banner length is 255 for SSH2 */
     char buffer[256] = {0};
     size_t len;
     int rc = SSH_ERROR;
 
-    banner = session->version == 1 ? CLIENTBANNER1 : CLIENTBANNER2;
-    terminator = session->version == 1 ? "\n" : "\r\n";
-
     if (server == 1) {
         if (session->opts.custombanner == NULL){
             len = strlen(banner);
@@ -211,11 +208,8 @@ int ssh_send_banner(ssh_session session, int server)
             goto end;
         }
 
-        /* SSH version 1 has a banner length of 128 only */
-        len = session->version == 1 ? 128 : 0;
-
         snprintf(buffer,
-                 sizeof(buffer) - len,
+                 sizeof(buffer),
                  "%s%s",
                  session->clientbanner,
                  terminator);
@@ -260,10 +254,13 @@ static int dh_handshake(ssh_session session) {
           break;
 #ifdef HAVE_ECDH
         case SSH_KEX_ECDH_SHA2_NISTP256:
+        case SSH_KEX_ECDH_SHA2_NISTP384:
+        case SSH_KEX_ECDH_SHA2_NISTP521:
           rc = ssh_client_ecdh_init(session);
           break;
 #endif
 #ifdef HAVE_CURVE25519
+        case SSH_KEX_CURVE25519_SHA256:
         case SSH_KEX_CURVE25519_SHA256_LIBSSH_ORG:
           rc = ssh_client_curve25519_init(session);
           break;
@@ -334,7 +331,7 @@ int ssh_service_request(ssh_session session, const char *service) {
       return SSH_ERROR;
   }
   session->auth_service_state=SSH_AUTH_SERVICE_SENT;
-  if (packet_send(session) == SSH_ERROR) {
+  if (ssh_packet_send(session) == SSH_ERROR) {
     ssh_set_error(session, SSH_FATAL,
         "Sending SSH2_MSG_SERVICE_REQUEST failed.");
       return SSH_ERROR;
@@ -359,8 +356,6 @@ pending:
     rc=SSH_AGAIN;
     break;
   case SSH_AUTH_SERVICE_NONE:
-  case SSH_AUTH_SERVICE_USER_SENT:
-    /* Invalid state, SSH1 specific */
     rc=SSH_ERROR;
     break;
   }
@@ -380,121 +375,86 @@ pending:
  * @brief A function to be called each time a step has been done in the
  * connection.
  */
-static void ssh_client_connection_callback(ssh_session session){
-	int ssh1,ssh2;
+static void ssh_client_connection_callback(ssh_session session)
+{
+    int rc;
 
-	switch(session->session_state){
-		case SSH_SESSION_STATE_NONE:
-		case SSH_SESSION_STATE_CONNECTING:
-			break;
-		case SSH_SESSION_STATE_SOCKET_CONNECTED:
-            /* If SSHv1 is disabled, we can send the banner immedietly */
-            if (session->opts.ssh1 == 0) {
-                ssh_set_fd_towrite(session);
-                ssh_send_banner(session, 0);
+    switch(session->session_state) {
+        case SSH_SESSION_STATE_NONE:
+        case SSH_SESSION_STATE_CONNECTING:
+            break;
+        case SSH_SESSION_STATE_SOCKET_CONNECTED:
+            ssh_set_fd_towrite(session);
+            ssh_send_banner(session, 0);
+
+            break;
+        case SSH_SESSION_STATE_BANNER_RECEIVED:
+            if (session->serverbanner == NULL) {
+                goto error;
             }
-			break;
-		case SSH_SESSION_STATE_BANNER_RECEIVED:
-		  if (session->serverbanner == NULL) {
-		    goto error;
-		  }
-		  set_status(session, 0.4f);
-		  SSH_LOG(SSH_LOG_RARE,
-		      "SSH server banner: %s", session->serverbanner);
+            set_status(session, 0.4f);
+            SSH_LOG(SSH_LOG_RARE,
+                    "SSH server banner: %s", session->serverbanner);
 
-		  /* Here we analyze the different protocols the server allows. */
-		  if (ssh_analyze_banner(session, 0, &ssh1, &ssh2) < 0) {
-		    goto error;
-		  }
-		  /* Here we decide which version of the protocol to use. */
-		  if (ssh2 && session->opts.ssh2) {
-		    session->version = 2;
-#ifdef WITH_SSH1
-		    } else if(ssh1 && session->opts.ssh1) {
-		    session->version = 1;
-#endif
-		    } else if(ssh1 && !session->opts.ssh1){
-#ifdef WITH_SSH1
-		    ssh_set_error(session, SSH_FATAL,
-		        "SSH-1 protocol not available (configure session to allow SSH-1)");
-		    goto error;
-#else
-		    ssh_set_error(session, SSH_FATAL,
-		        "SSH-1 protocol not available (libssh compiled without SSH-1 support)");
-		    goto error;
-#endif
-		  } else {
-		    ssh_set_error(session, SSH_FATAL,
-		        "No version of SSH protocol usable (banner: %s)",
-		        session->serverbanner);
-		    goto error;
-		  }
-		  /* from now, the packet layer is handling incoming packets */
-		  if(session->version==2)
-		    session->socket_callbacks.data=ssh_packet_socket_callback;
-#ifdef WITH_SSH1
-		  else
-		    session->socket_callbacks.data=ssh_packet_socket_callback1;
-#endif
-		  ssh_packet_set_default_callbacks(session);
-		  session->session_state=SSH_SESSION_STATE_INITIAL_KEX;
-          if (session->opts.ssh1 == 1) {
-              ssh_send_banner(session, 0);
-          }
-		  set_status(session, 0.5f);
-		  break;
-		case SSH_SESSION_STATE_INITIAL_KEX:
-		/* TODO: This state should disappear in favor of get_key handle */
-#ifdef WITH_SSH1
-			if(session->version==1){
-				if (ssh_get_kex1(session) < 0)
-					goto error;
-				set_status(session,0.6f);
-				session->connected = 1;
-				break;
-			}
-#endif
-			break;
-		case SSH_SESSION_STATE_KEXINIT_RECEIVED:
-			set_status(session,0.6f);
-			ssh_list_kex(&session->next_crypto->server_kex);
-			if (set_client_kex(session) < 0) {
-				goto error;
-			}
-			if (ssh_kex_select_methods(session) == SSH_ERROR)
-			    goto error;
-			if (ssh_send_kex(session, 0) < 0) {
-				goto error;
-			}
-			set_status(session,0.8f);
-			session->session_state=SSH_SESSION_STATE_DH;
-			if (dh_handshake(session) == SSH_ERROR) {
-				goto error;
-			}
+            /* Here we analyze the different protocols the server allows. */
+            rc = ssh_analyze_banner(session, 0);
+            if (rc < 0) {
+                ssh_set_error(session, SSH_FATAL,
+                        "No version of SSH protocol usable (banner: %s)",
+                        session->serverbanner);
+                goto error;
+            }
+
+            ssh_packet_register_socket_callback(session, session->socket);
+
+            ssh_packet_set_default_callbacks(session);
+            session->session_state = SSH_SESSION_STATE_INITIAL_KEX;
+            set_status(session, 0.5f);
+
+            break;
+        case SSH_SESSION_STATE_INITIAL_KEX:
+            /* TODO: This state should disappear in favor of get_key handle */
+            break;
+        case SSH_SESSION_STATE_KEXINIT_RECEIVED:
+            set_status(session,0.6f);
+            ssh_list_kex(&session->next_crypto->server_kex);
+            if (ssh_set_client_kex(session) < 0) {
+                goto error;
+            }
+            if (ssh_kex_select_methods(session) == SSH_ERROR)
+                goto error;
+            if (ssh_send_kex(session, 0) < 0) {
+                goto error;
+            }
+            set_status(session,0.8f);
+            session->session_state=SSH_SESSION_STATE_DH;
+            if (dh_handshake(session) == SSH_ERROR) {
+                goto error;
+            }
             /* FALL THROUGH */
-		case SSH_SESSION_STATE_DH:
-			if(session->dh_handshake_state==DH_STATE_FINISHED){
-				set_status(session,1.0f);
-				session->connected = 1;
-				if (session->flags & SSH_SESSION_FLAG_AUTHENTICATED)
-				    session->session_state = SSH_SESSION_STATE_AUTHENTICATED;
-				else
-				    session->session_state=SSH_SESSION_STATE_AUTHENTICATING;
-			}
-			break;
-		case SSH_SESSION_STATE_AUTHENTICATING:
-			break;
-		case SSH_SESSION_STATE_ERROR:
-			goto error;
-		default:
-			ssh_set_error(session,SSH_FATAL,"Invalid state %d",session->session_state);
-	}
+        case SSH_SESSION_STATE_DH:
+            if(session->dh_handshake_state==DH_STATE_FINISHED){
+                set_status(session,1.0f);
+                session->connected = 1;
+                if (session->flags & SSH_SESSION_FLAG_AUTHENTICATED)
+                    session->session_state = SSH_SESSION_STATE_AUTHENTICATED;
+                else
+                    session->session_state=SSH_SESSION_STATE_AUTHENTICATING;
+            }
+            break;
+        case SSH_SESSION_STATE_AUTHENTICATING:
+            break;
+        case SSH_SESSION_STATE_ERROR:
+            goto error;
+        default:
+            ssh_set_error(session,SSH_FATAL,"Invalid state %d",session->session_state);
+    }
 
-	return;
+    return;
 error:
-	ssh_socket_close(session->socket);
-	session->alive = 0;
-	session->session_state=SSH_SESSION_STATE_ERROR;
+    ssh_socket_close(session->socket);
+    session->alive = 0;
+    session->session_state=SSH_SESSION_STATE_ERROR;
 
 }
 
@@ -545,9 +505,6 @@ int ssh_connect(ssh_session session) {
   session->alive = 0;
   session->client = 1;
 
-  if (ssh_init() < 0) {
-    return SSH_ERROR;
-  }
   if (session->opts.fd == SSH_INVALID_SOCKET &&
       session->opts.host == NULL &&
       session->opts.ProxyCommand == NULL) {
@@ -693,19 +650,22 @@ void ssh_disconnect(ssh_session session) {
 
   if (session->socket != NULL && ssh_socket_is_open(session->socket)) {
     rc = ssh_buffer_pack(session->out_buffer,
-                         "bds",
+                         "bdss",
                          SSH2_MSG_DISCONNECT,
                          SSH2_DISCONNECT_BY_APPLICATION,
-                         "Bye Bye");
+                         "Bye Bye",
+                         ""); /* language tag */
     if (rc != SSH_OK){
       ssh_set_error_oom(session);
       goto error;
     }
 
-    packet_send(session);
+    ssh_packet_send(session);
     ssh_socket_close(session->socket);
   }
 error:
+  session->recv_seq = 0;
+  session->send_seq = 0;
   session->alive = 0;
   if (session->socket != NULL){
     ssh_socket_reset(session->socket);
@@ -721,6 +681,13 @@ error:
     crypto_free(session->current_crypto);
     session->current_crypto=NULL;
   }
+  if (session->next_crypto) {
+    crypto_free(session->next_crypto);
+    session->next_crypto = crypto_new();
+    if (session->next_crypto == NULL) {
+      ssh_set_error_oom(session);
+    }
+  }
   if (session->in_buffer) {
     ssh_buffer_reinit(session->in_buffer);
   }
@@ -754,10 +721,10 @@ error:
 }
 
 const char *ssh_copyright(void) {
-    return SSH_STRINGIFY(LIBSSH_VERSION) " (c) 2003-2014 Aris Adamantiadis, Andreas Schneider, "
-    "and libssh contributors. Distributed under the LGPL, please refer to COPYING "
-    "file for information about your rights";
+    return SSH_STRINGIFY(LIBSSH_VERSION) " (c) 2003-2018 "
+           "Aris Adamantiadis, Andreas Schneider "
+           "and libssh contributors. "
+           "Distributed under the LGPL, please refer to COPYING "
+           "file for information about your rights";
 }
 /** @} */
-
-/* vim: set ts=4 sw=4 et cindent: */
