@@ -74,6 +74,7 @@
 #include <sys/socket.h>
 #include <sys/select.h>
 #include <netinet/in.h>
+#include <netinet/tcp.h>
 
 #endif /* _WIN32 */
 
@@ -203,7 +204,7 @@ static int ssh_connect_ai_timeout(ssh_session session, const char *host,
   }
 
   /* s is connected ? */
-  SSH_LOG(SSH_LOG_PACKET, "Socket connected with timeout\n");
+  SSH_LOG(SSH_LOG_PACKET, "Socket connected with timeout");
   ret = ssh_socket_set_blocking(s);
   if (ret < 0) {
       ssh_set_error(session, SSH_FATAL,
@@ -216,6 +217,12 @@ static int ssh_connect_ai_timeout(ssh_session session, const char *host,
   return s;
 }
 
+static int set_tcp_nodelay(socket_t socket)
+{
+    int opt = 1;
+    return setsockopt(socket, IPPROTO_TCP, TCP_NODELAY, &opt, sizeof(opt));
+}
+
 /**
  * @internal
  *
@@ -252,7 +259,7 @@ socket_t ssh_connect_host(ssh_session session, const char *host,
       struct addrinfo *bind_ai;
       struct addrinfo *bind_itr;
 
-      SSH_LOG(SSH_LOG_PACKET, "Resolving %s\n", bind_addr);
+      SSH_LOG(SSH_LOG_PACKET, "Resolving %s", bind_addr);
 
       rc = getai(bind_addr, 0, &bind_ai);
       if (rc != 0) {
@@ -346,7 +353,7 @@ socket_t ssh_connect_host_nonblocking(ssh_session session, const char *host,
       struct addrinfo *bind_ai;
       struct addrinfo *bind_itr;
 
-      SSH_LOG(SSH_LOG_PACKET, "Resolving %s\n", bind_addr);
+      SSH_LOG(SSH_LOG_PACKET, "Resolving %s", bind_addr);
 
       rc = getai(bind_addr, 0, &bind_ai);
       if (rc != 0) {
@@ -387,6 +394,18 @@ socket_t ssh_connect_host_nonblocking(ssh_session session, const char *host,
         continue;
     }
 
+    if (session->opts.nodelay) {
+        /* For winsock, socket options are only effective before connect */
+        rc = set_tcp_nodelay(s);
+        if (rc < 0) {
+            ssh_set_error(session, SSH_FATAL,
+                "Failed to set TCP_NODELAY on socket: %s", strerror(errno));
+            ssh_connect_socket_close(s);
+            s = -1;
+            continue;
+        }
+    }
+
     errno = 0;
     rc = connect(s, itr->ai_addr, itr->ai_addrlen);
     if (rc == -1 && (errno != 0) && (errno != EINPROGRESS)) {
@@ -518,5 +537,3 @@ out:
 }
 
 /** @} */
-
-/* vim: set ts=4 sw=4 et cindent: */
