@@ -115,13 +115,13 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_gssapi_token){
  * @param[in] oid the OID that was selected for authentication
  */
 static int ssh_gssapi_send_response(ssh_session session, ssh_string oid){
-    if (buffer_add_u8(session->out_buffer, SSH2_MSG_USERAUTH_GSSAPI_RESPONSE) < 0 ||
-            buffer_add_ssh_string(session->out_buffer,oid) < 0) {
+    if (ssh_buffer_add_u8(session->out_buffer, SSH2_MSG_USERAUTH_GSSAPI_RESPONSE) < 0 ||
+            ssh_buffer_add_ssh_string(session->out_buffer,oid) < 0) {
         ssh_set_error_oom(session);
         return SSH_ERROR;
     }
 
-    packet_send(session);
+    ssh_packet_send(session);
     SSH_LOG(SSH_LOG_PACKET,
             "Sent SSH_MSG_USERAUTH_GSSAPI_RESPONSE");
     return SSH_OK;
@@ -218,7 +218,7 @@ int ssh_gssapi_handle_userauth(ssh_session session, const char *user, uint32_t n
     maj_stat = gss_indicate_mechs(&min_stat, &supported);
     for (i=0; i < supported->count; ++i){
         ptr = ssh_get_hexa(supported->elements[i].elements, supported->elements[i].length);
-        SSH_LOG(SSH_LOG_DEBUG, "Supported mech %d: %s\n", i, ptr);
+        SSH_LOG(SSH_LOG_DEBUG, "Supported mech %d: %s", i, ptr);
         free(ptr);
     }
 
@@ -344,7 +344,7 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_gssapi_token_server){
         ssh_set_error(session, SSH_FATAL, "Received SSH_MSG_USERAUTH_GSSAPI_TOKEN in invalid state");
         return SSH_PACKET_USED;
     }
-    token = buffer_get_ssh_string(packet);
+    token = ssh_buffer_get_ssh_string(packet);
 
     if (token == NULL){
         ssh_set_error(session, SSH_REQUEST_DENIED, "ssh_packet_userauth_gssapi_token: invalid packet");
@@ -370,7 +370,7 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_gssapi_token_server){
                 ssh_set_error_oom(session);
                 return SSH_PACKET_USED;
             }
-            packet_send(session);
+            ssh_packet_send(session);
             ssh_string_free(out_token);
         } else {
             session->gssapi->state = SSH_GSSAPI_STATE_RCV_MIC;
@@ -415,7 +415,7 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_gssapi_token_server){
                         SSH2_MSG_USERAUTH_GSSAPI_TOKEN,
                         output_token.length,
                         (size_t)output_token.length, output_token.value);
-        packet_send(session);
+        ssh_packet_send(session);
     }
     if(maj_stat == GSS_S_COMPLETE){
         session->gssapi->state = SSH_GSSAPI_STATE_RCV_MIC;
@@ -466,7 +466,7 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_gssapi_mic)
     (void)type;
 
     SSH_LOG(SSH_LOG_PACKET,"Received SSH_MSG_USERAUTH_GSSAPI_MIC");
-    mic_token = buffer_get_ssh_string(packet);
+    mic_token = ssh_buffer_get_ssh_string(packet);
     if (mic_token == NULL) {
         ssh_set_error(session, SSH_FATAL, "Missing MIC in packet");
         goto error;
@@ -484,14 +484,14 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_gssapi_mic)
     }
     if (ssh_callbacks_exists(session->server_callbacks, gssapi_verify_mic_function)){
         int rc = session->server_callbacks->gssapi_verify_mic_function(session, mic_token,
-                ssh_buffer_get_begin(mic_buffer), ssh_buffer_get_len(mic_buffer),
+                ssh_buffer_get(mic_buffer), ssh_buffer_get_len(mic_buffer),
                 session->server_callbacks->userdata);
         if (rc != SSH_OK) {
             goto error;
         }
     } else {
         mic_buf.length = ssh_buffer_get_len(mic_buffer);
-        mic_buf.value = ssh_buffer_get_begin(mic_buffer);
+        mic_buf.value = ssh_buffer_get(mic_buffer);
         mic_token_buf.length = ssh_string_len(mic_token);
         mic_token_buf.value = ssh_string_data(mic_token);
 
@@ -592,14 +592,14 @@ static int ssh_gssapi_send_auth_mic(ssh_session session, ssh_string *oid_set, in
     }
 
     for (i=0; i<n_oid; ++i){
-        rc = buffer_add_ssh_string(session->out_buffer, oid_set[i]);
+        rc = ssh_buffer_add_ssh_string(session->out_buffer, oid_set[i]);
         if (rc < 0) {
             goto fail;
         }
     }
 
     session->auth_state = SSH_AUTH_STATE_GSSAPI_REQUEST_SENT;
-    return packet_send(session);
+    return ssh_packet_send(session);
 fail:
     ssh_buffer_reinit(session->out_buffer);
     return SSH_ERROR;
@@ -664,7 +664,7 @@ static int ssh_gssapi_match(ssh_session session, gss_OID_set *valid_oids)
         if (maj_stat == GSS_S_COMPLETE && lifetime > 0) {
             gss_add_oid_set_member(&min_stat, oid, valid_oids);
             ptr = ssh_get_hexa(oid->elements, oid->length);
-            SSH_LOG(SSH_LOG_DEBUG, "GSSAPI valid oid %d : %s\n", i, ptr);
+            SSH_LOG(SSH_LOG_DEBUG, "GSSAPI valid oid %d : %s", i, ptr);
             SAFE_FREE(ptr);
         }
     }
@@ -763,20 +763,27 @@ int ssh_gssapi_auth_mic(ssh_session session){
 }
 
 static gss_OID ssh_gssapi_oid_from_string(ssh_string oid_s){
-    gss_OID ret = malloc(sizeof (gss_OID_desc));
+    gss_OID ret;
     unsigned char *data = ssh_string_data(oid_s);
     size_t len = ssh_string_len(oid_s);
-    if(len > 256 || len <= 2){
+
+    ret = malloc(sizeof(gss_OID_desc));
+    if (ret == NULL) {
+        return NULL;
+    }
+
+    if (len > 256 || len <= 2) {
         SAFE_FREE(ret);
         return NULL;
     }
-    if(data[0] != SSH_OID_TAG || data[1] != len - 2){
+    if (data[0] != SSH_OID_TAG || data[1] != len - 2) {
         SAFE_FREE(ret);
         return NULL;
     }
     ret->elements = malloc(len - 2);
     memcpy(ret->elements, &data[2], len-2);
     ret->length = len-2;
+
     return ret;
 }
 
@@ -792,18 +799,19 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_gssapi_response){
     SSH_LOG(SSH_LOG_PACKET, "Received SSH_USERAUTH_GSSAPI_RESPONSE");
     if (session->auth_state != SSH_AUTH_STATE_GSSAPI_REQUEST_SENT){
         ssh_set_error(session, SSH_FATAL, "Invalid state in ssh_packet_userauth_gssapi_response");
-        return SSH_PACKET_USED;
+        goto error;
     }
-    oid_s = buffer_get_ssh_string(packet);
+
+    oid_s = ssh_buffer_get_ssh_string(packet);
     if (!oid_s){
         ssh_set_error(session, SSH_FATAL, "Missing OID");
-        return SSH_PACKET_USED;
+        goto error;
     }
     session->gssapi->client.oid = ssh_gssapi_oid_from_string(oid_s);
     ssh_string_free(oid_s);
     if (!session->gssapi->client.oid) {
         ssh_set_error(session, SSH_FATAL, "Invalid OID");
-        return SSH_PACKET_USED;
+        goto error;
     }
 
     session->gssapi->client.flags = GSS_C_MUTUAL_FLAG | GSS_C_INTEG_FLAG;
@@ -825,21 +833,27 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_gssapi_response){
                              "Initializing gssapi context",
                              maj_stat,
                              min_stat);
-        return SSH_PACKET_USED;
+        goto error;
     }
     if (output_token.length != 0){
         hexa = ssh_get_hexa(output_token.value, output_token.length);
-        SSH_LOG(SSH_LOG_PACKET, "GSSAPI: sending token %s",hexa);
+        SSH_LOG(SSH_LOG_PACKET, "GSSAPI: sending token %s", hexa);
         SAFE_FREE(hexa);
         ssh_buffer_pack(session->out_buffer,
                         "bdP",
                         SSH2_MSG_USERAUTH_GSSAPI_TOKEN,
                         output_token.length,
                         (size_t)output_token.length, output_token.value);
-        packet_send(session);
+        ssh_packet_send(session);
         session->auth_state = SSH_AUTH_STATE_GSSAPI_TOKEN;
     }
     return SSH_PACKET_USED;
+
+error:
+    session->auth_state = SSH_AUTH_STATE_ERROR;
+    ssh_gssapi_free(session);
+    session->gssapi = NULL;
+    return SSH_PACKET_USED;
 }
 
 static int ssh_gssapi_send_mic(ssh_session session){
@@ -857,9 +871,10 @@ static int ssh_gssapi_send_mic(ssh_session session){
         return SSH_ERROR;
     }
     mic_buf.length = ssh_buffer_get_len(mic_buffer);
-    mic_buf.value = ssh_buffer_get_begin(mic_buffer);
+    mic_buf.value = ssh_buffer_get(mic_buffer);
 
-    maj_stat = gss_get_mic(&min_stat,session->gssapi->ctx, GSS_C_QOP_DEFAULT, &mic_buf, &mic_token_buf);
+    maj_stat = gss_get_mic(&min_stat,session->gssapi->ctx, GSS_C_QOP_DEFAULT,
+                           &mic_buf, &mic_token_buf);
     if (GSS_ERROR(maj_stat)){
         ssh_buffer_free(mic_buffer);
         ssh_gssapi_log_error(SSH_LOG_PROTOCOL,
@@ -880,7 +895,7 @@ static int ssh_gssapi_send_mic(ssh_session session){
         return SSH_ERROR;
     }
 
-    return packet_send(session);
+    return ssh_packet_send(session);
 }
 
 SSH_PACKET_CALLBACK(ssh_packet_userauth_gssapi_token_client){
@@ -892,16 +907,19 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_gssapi_token_client){
     (void)type;
 
     SSH_LOG(SSH_LOG_PACKET,"Received SSH_MSG_USERAUTH_GSSAPI_TOKEN");
-    if (!session->gssapi || session->auth_state != SSH_AUTH_STATE_GSSAPI_TOKEN){
-        ssh_set_error(session, SSH_FATAL, "Received SSH_MSG_USERAUTH_GSSAPI_TOKEN in invalid state");
-        return SSH_PACKET_USED;
+    if (!session->gssapi || session->auth_state != SSH_AUTH_STATE_GSSAPI_TOKEN) {
+        ssh_set_error(session, SSH_FATAL,
+                      "Received SSH_MSG_USERAUTH_GSSAPI_TOKEN in invalid state");
+        goto error;
     }
-    token = buffer_get_ssh_string(packet);
+    token = ssh_buffer_get_ssh_string(packet);
 
     if (token == NULL){
-        ssh_set_error(session, SSH_REQUEST_DENIED, "ssh_packet_userauth_gssapi_token: invalid packet");
-        return SSH_PACKET_USED;
+        ssh_set_error(session, SSH_REQUEST_DENIED,
+                      "ssh_packet_userauth_gssapi_token: invalid packet");
+        goto error;
     }
+
     hexa = ssh_get_hexa(ssh_string_data(token),ssh_string_len(token));
     SSH_LOG(SSH_LOG_PACKET, "GSSAPI Token : %s",hexa);
     SAFE_FREE(hexa);
@@ -926,12 +944,10 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_gssapi_token_client){
                              "Gssapi error",
                              maj_stat,
                              min_stat);
-        ssh_gssapi_free(session);
-        session->gssapi=NULL;
-        return SSH_PACKET_USED;
+        goto error;
     }
 
-    if (output_token.length != 0){
+    if (output_token.length != 0) {
         hexa = ssh_get_hexa(output_token.value, output_token.length);
         SSH_LOG(SSH_LOG_PACKET, "GSSAPI: sending token %s",hexa);
         SAFE_FREE(hexa);
@@ -940,11 +956,19 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_gssapi_token_client){
                         SSH2_MSG_USERAUTH_GSSAPI_TOKEN,
                         output_token.length,
                         (size_t)output_token.length, output_token.value);
-        packet_send(session);
+        ssh_packet_send(session);
     }
-    if(maj_stat == GSS_S_COMPLETE){
+
+    if (maj_stat == GSS_S_COMPLETE) {
+        session->auth_state = SSH_AUTH_STATE_NONE;
         ssh_gssapi_send_mic(session);
-        session->auth_state = SSH_AUTH_STATE_GSSAPI_MIC_SENT;
     }
+
+    return SSH_PACKET_USED;
+
+error:
+    session->auth_state = SSH_AUTH_STATE_ERROR;
+    ssh_gssapi_free(session);
+    session->gssapi = NULL;
     return SSH_PACKET_USED;
 }
