@@ -58,8 +58,8 @@ static z_stream *initcompress(ssh_session session, int level) {
 
 static ssh_buffer gzip_compress(ssh_session session,ssh_buffer source,int level){
   z_stream *zout = session->current_crypto->compress_out_ctx;
-  void *in_ptr = buffer_get_rest(source);
-  unsigned long in_size = buffer_get_rest_len(source);
+  void *in_ptr = ssh_buffer_get(source);
+  unsigned long in_size = ssh_buffer_get_len(source);
   ssh_buffer dest = NULL;
   unsigned char out_buf[BLOCKSIZE] = {0};
   unsigned long len;
@@ -90,7 +90,7 @@ static ssh_buffer gzip_compress(ssh_session session,ssh_buffer source,int level)
       return NULL;
     }
     len = BLOCKSIZE - zout->avail_out;
-    if (buffer_add_data(dest, out_buf, len) < 0) {
+    if (ssh_buffer_add_data(dest, out_buf, len) < 0) {
       ssh_buffer_free(dest);
       return NULL;
     }
@@ -108,12 +108,12 @@ int compress_buffer(ssh_session session, ssh_buffer buf) {
     return -1;
   }
 
-  if (buffer_reinit(buf) < 0) {
+  if (ssh_buffer_reinit(buf) < 0) {
     ssh_buffer_free(dest);
     return -1;
   }
 
-  if (buffer_add_data(buf, buffer_get_rest(dest), buffer_get_rest_len(dest)) < 0) {
+  if (ssh_buffer_add_data(buf, ssh_buffer_get(dest), ssh_buffer_get_len(dest)) < 0) {
     ssh_buffer_free(dest);
     return -1;
   }
@@ -147,8 +147,8 @@ static z_stream *initdecompress(ssh_session session) {
 
 static ssh_buffer gzip_decompress(ssh_session session, ssh_buffer source, size_t maxlen) {
   z_stream *zin = session->current_crypto->compress_in_ctx;
-  void *in_ptr = buffer_get_rest(source);
-  unsigned long in_size = buffer_get_rest_len(source);
+  void *in_ptr = ssh_buffer_get(source);
+  unsigned long in_size = ssh_buffer_get_len(source);
   unsigned char out_buf[BLOCKSIZE] = {0};
   ssh_buffer dest = NULL;
   unsigned long len;
@@ -181,11 +181,11 @@ static ssh_buffer gzip_decompress(ssh_session session, ssh_buffer source, size_t
     }
 
     len = BLOCKSIZE - zin->avail_out;
-    if (buffer_add_data(dest,out_buf,len) < 0) {
+    if (ssh_buffer_add_data(dest,out_buf,len) < 0) {
       ssh_buffer_free(dest);
       return NULL;
     }
-    if (buffer_get_rest_len(dest) > maxlen){
+    if (ssh_buffer_get_len(dest) > maxlen){
       /* Size of packet exceeded, avoid a denial of service attack */
       ssh_buffer_free(dest);
       return NULL;
@@ -204,12 +204,12 @@ int decompress_buffer(ssh_session session,ssh_buffer buf, size_t maxlen){
     return -1;
   }
 
-  if (buffer_reinit(buf) < 0) {
+  if (ssh_buffer_reinit(buf) < 0) {
     ssh_buffer_free(dest);
     return -1;
   }
 
-  if (buffer_add_data(buf, buffer_get_rest(dest), buffer_get_rest_len(dest)) < 0) {
+  if (ssh_buffer_add_data(buf, ssh_buffer_get(dest), ssh_buffer_get_len(dest)) < 0) {
     ssh_buffer_free(dest);
     return -1;
   }
@@ -217,5 +217,3 @@ int decompress_buffer(ssh_session session,ssh_buffer buf, size_t maxlen){
   ssh_buffer_free(dest);
   return 0;
 }
-
-/* vim: set ts=2 sw=2 et cindent: */
