@@ -36,12 +36,19 @@
 #include "libssh/string.h"
 #include "libssh/curve25519.h"
 #include "libssh/knownhosts.h"
+#include "libssh/misc.h"
 
 #ifdef HAVE_LIBGCRYPT
 # define BLOWFISH "blowfish-cbc,"
 # define AES "aes256-ctr,aes192-ctr,aes128-ctr,aes256-cbc,aes192-cbc,aes128-cbc,"
 # define DES "3des-cbc"
-# define DES_SUPPORTED "3des-cbc,des-cbc-ssh1"
+# define DES_SUPPORTED "3des-cbc"
+
+#elif defined HAVE_LIBMBEDCRYPTO
+# define BLOWFISH "blowfish-cbc,"
+# define AES "aes256-ctr,aes192-ctr,aes128-ctr,aes256-cbc,aes192-cbc,aes128-cbc,"
+# define DES "3des-cbc"
+# define DES_SUPPORTED "3des-cbc"
 
 #elif defined(HAVE_LIBCRYPTO)
 
@@ -62,7 +69,7 @@
 # endif /* HAVE_OPENSSL_AES_H */
 
 # define DES "3des-cbc"
-# define DES_SUPPORTED "3des-cbc,des-cbc-ssh1"
+# define DES_SUPPORTED "3des-cbc"
 #endif /* HAVE_LIBCRYPTO */
 
 #ifdef WITH_ZLIB
@@ -72,19 +79,25 @@
 #endif
 
 #ifdef HAVE_CURVE25519
-#define CURVE25519 "curve25519-sha256@libssh.org,"
+#define CURVE25519 "curve25519-sha256,curve25519-sha256@libssh.org,"
 #else
 #define CURVE25519 ""
 #endif
 
 #ifdef HAVE_ECDH
-#define ECDH "ecdh-sha2-nistp256,"
+#define ECDH "ecdh-sha2-nistp256,ecdh-sha2-nistp384,ecdh-sha2-nistp521,"
 #define HOSTKEYS "ssh-ed25519,ecdsa-sha2-nistp256,ecdsa-sha2-nistp384,ecdsa-sha2-nistp521,ssh-rsa,ssh-dss"
 #else
+#ifdef HAVE_DSA
 #define HOSTKEYS "ssh-ed25519,ssh-rsa,ssh-dss"
+#else
+#define HOSTKEYS "ssh-ed25519,ssh-rsa"
+#endif
 #define ECDH ""
 #endif
 
+#define CHACHA20 "chacha20-poly1305@openssh.com,"
+
 #define KEY_EXCHANGE CURVE25519 ECDH "diffie-hellman-group14-sha1,diffie-hellman-group1-sha1"
 #define KEX_METHODS_SIZE 10
 
@@ -107,8 +120,8 @@ static const char *default_methods[] = {
 static const char *supported_methods[] = {
   KEY_EXCHANGE,
   HOSTKEYS,
-  AES BLOWFISH DES_SUPPORTED,
-  AES BLOWFISH DES_SUPPORTED,
+  CHACHA20 AES BLOWFISH DES_SUPPORTED,
+  CHACHA20 AES BLOWFISH DES_SUPPORTED,
   "hmac-sha2-256,hmac-sha2-512,hmac-sha1",
   "hmac-sha2-256,hmac-sha2-512,hmac-sha1",
   ZLIB,
@@ -154,7 +167,7 @@ static char **tokenize(const char *chain){
         ptr++;
     }
     /* now n contains the number of tokens, the first possibly empty if the list was empty too e.g. "" */
-    tokens=malloc(sizeof(char *) * (n+1) ); /* +1 for the null */
+    tokens = calloc(n + 1, sizeof(char *)); /* +1 for the null */
     if (tokens == NULL) {
       SAFE_FREE(tmp);
       return NULL;
@@ -172,7 +185,7 @@ static char **tokenize(const char *chain){
 
 /* same as tokenize(), but with spaces instead of ',' */
 /* TODO FIXME rewrite me! */
-char **space_tokenize(const char *chain){
+char **ssh_space_tokenize(const char *chain){
     char **tokens;
     int n=1;
     int i=0;
@@ -198,7 +211,7 @@ char **space_tokenize(const char *chain){
         ptr++;
     }
     /* now n contains the number of tokens, the first possibly empty if the list was empty too e.g. "" */
-    tokens = malloc(sizeof(char *) * (n + 1)); /* +1 for the null */
+    tokens = calloc(n + 1, sizeof(char *)); /* +1 for the null */
     if (tokens == NULL) {
       SAFE_FREE(tmp);
       return NULL;
@@ -281,6 +294,71 @@ char *ssh_find_matching(const char *available_d, const char *preferred_d){
     return NULL;
 }
 
+static char *ssh_find_all_matching(const char *available_d,
+                                   const char *preferred_d)
+{
+    char **tok_available, **tok_preferred;
+    int i_avail, i_pref;
+    char *ret;
+    unsigned max, len, pos = 0;
+
+    if ((available_d == NULL) || (preferred_d == NULL)) {
+        return NULL; /* don't deal with null args */
+    }
+
+    max = MAX(strlen(available_d), strlen(preferred_d));
+
+    ret = malloc(max+1);
+    if (ret == NULL) {
+      return NULL;
+    }
+    ret[0] = 0;
+
+    tok_available = tokenize(available_d);
+    if (tok_available == NULL) {
+        SAFE_FREE(ret);
+        return NULL;
+    }
+
+    tok_preferred = tokenize(preferred_d);
+    if (tok_preferred == NULL) {
+        SAFE_FREE(ret);
+        SAFE_FREE(tok_available[0]);
+        SAFE_FREE(tok_available);
+        return NULL;
+    }
+
+    for (i_pref = 0; tok_preferred[i_pref] ; ++i_pref) {
+        for (i_avail = 0; tok_available[i_avail]; ++i_avail) {
+            int cmp = strcmp(tok_available[i_avail],tok_preferred[i_pref]);
+            if (cmp == 0) {
+                /* match */
+                if (pos != 0) {
+                    ret[pos] = ',';
+                    pos++;
+                }
+
+                len = strlen(tok_available[i_avail]);
+                memcpy(&ret[pos], tok_available[i_avail], len);
+                pos += len;
+                ret[pos] = '\0';
+            }
+        }
+    }
+
+    if (ret[0] == '\0') {
+        SAFE_FREE(ret);
+        ret = NULL;
+    }
+
+    SAFE_FREE(tok_available[0]);
+    SAFE_FREE(tok_preferred[0]);
+    SAFE_FREE(tok_available);
+    SAFE_FREE(tok_preferred);
+
+    return ret;
+}
+
 /**
  * @internal
  * @brief returns whether the first client key exchange algorithm or
@@ -346,25 +424,25 @@ SSH_PACKET_CALLBACK(ssh_packet_kexinit){
     }
 
     if (server_kex) {
-        rc = buffer_get_data(packet,session->next_crypto->client_kex.cookie, 16);
+        rc = ssh_buffer_get_data(packet,session->next_crypto->client_kex.cookie, 16);
         if (rc != 16) {
             ssh_set_error(session, SSH_FATAL, "ssh_packet_kexinit: no cookie in packet");
             goto error;
         }
 
-        rc = hashbufin_add_cookie(session, session->next_crypto->client_kex.cookie);
+        rc = ssh_hashbufin_add_cookie(session, session->next_crypto->client_kex.cookie);
         if (rc < 0) {
             ssh_set_error(session, SSH_FATAL, "ssh_packet_kexinit: adding cookie failed");
             goto error;
         }
     } else {
-        rc = buffer_get_data(packet,session->next_crypto->server_kex.cookie, 16);
+        rc = ssh_buffer_get_data(packet,session->next_crypto->server_kex.cookie, 16);
         if (rc != 16) {
             ssh_set_error(session, SSH_FATAL, "ssh_packet_kexinit: no cookie in packet");
             goto error;
         }
 
-        rc = hashbufin_add_cookie(session, session->next_crypto->server_kex.cookie);
+        rc = ssh_hashbufin_add_cookie(session, session->next_crypto->server_kex.cookie);
         if (rc < 0) {
             ssh_set_error(session, SSH_FATAL, "ssh_packet_kexinit: adding cookie failed");
             goto error;
@@ -372,12 +450,12 @@ SSH_PACKET_CALLBACK(ssh_packet_kexinit){
     }
 
     for (i = 0; i < KEX_METHODS_SIZE; i++) {
-        str = buffer_get_ssh_string(packet);
+        str = ssh_buffer_get_ssh_string(packet);
         if (str == NULL) {
           goto error;
         }
 
-        rc = buffer_add_ssh_string(session->in_hashbuf, str);
+        rc = ssh_buffer_add_ssh_string(session->in_hashbuf, str);
         if (rc < 0) {
             ssh_set_error(session, SSH_FATAL, "Error adding string in hash buffer");
             goto error;
@@ -414,17 +492,17 @@ SSH_PACKET_CALLBACK(ssh_packet_kexinit){
      * 'make_sessionid').
      */
     if (server_kex) {
-        rc = buffer_get_u8(packet, &first_kex_packet_follows);
+        rc = ssh_buffer_get_u8(packet, &first_kex_packet_follows);
         if (rc != 1) {
             goto error;
         }
 
-        rc = buffer_add_u8(session->in_hashbuf, first_kex_packet_follows);
+        rc = ssh_buffer_add_u8(session->in_hashbuf, first_kex_packet_follows);
         if (rc < 0) {
             goto error;
         }
 
-        rc = buffer_add_u32(session->in_hashbuf, kexinit_reserved);
+        rc = ssh_buffer_add_u32(session->in_hashbuf, kexinit_reserved);
         if (rc < 0) {
             goto error;
         }
@@ -487,7 +565,8 @@ void ssh_list_kex(struct ssh_kex_struct *kex) {
  * @returns a cstring containing a comma-separated list of hostkey methods.
  *          NULL if no method matches
  */
-static char *ssh_client_select_hostkeys(ssh_session session){
+static char *ssh_client_select_hostkeys(ssh_session session)
+{
     char methods_buffer[128]={0};
     static const char *preferred_hostkeys[] = {
         "ssh-ed25519",
@@ -495,56 +574,85 @@ static char *ssh_client_select_hostkeys(ssh_session session){
         "ecdsa-sha2-nistp384",
         "ecdsa-sha2-nistp256",
         "ssh-rsa",
+#ifdef HAVE_DSA
         "ssh-dss",
-        "ssh-rsa1",
+#endif
         NULL
     };
-    char **methods;
-    int i,j;
-    int needcoma=0;
+    struct ssh_list *algo_list = NULL;
+    struct ssh_iterator *it = NULL;
+    size_t algo_count;
+    int needcomma = 0;
+    int i;
 
-    methods = ssh_knownhosts_algorithms(session);
-	if (methods == NULL || methods[0] == NULL){
-		SAFE_FREE(methods);
-		return NULL;
-	}
+    algo_list = ssh_known_hosts_get_algorithms(session);
+    if (algo_list == NULL) {
+        return NULL;
+    }
 
-	for (i=0;preferred_hostkeys[i] != NULL; ++i){
-		for (j=0; methods[j] != NULL; ++j){
-			if(strcmp(preferred_hostkeys[i], methods[j]) == 0){
-				if (verify_existing_algo(SSH_HOSTKEYS, methods[j])){
-					if(needcoma)
-						strncat(methods_buffer,",",sizeof(methods_buffer)-strlen(methods_buffer)-1);
-					strncat(methods_buffer, methods[j], sizeof(methods_buffer)-strlen(methods_buffer)-1);
-					needcoma = 1;
-				}
-			}
-		}
-	}
-	for(i=0;methods[i]!= NULL; ++i){
-		SAFE_FREE(methods[i]);
-	}
-	SAFE_FREE(methods);
+    algo_count = ssh_list_count(algo_list);
+    if (algo_count == 0) {
+        ssh_list_free(algo_list);
+        return NULL;
+    }
 
-	if(strlen(methods_buffer) > 0){
-		SSH_LOG(SSH_LOG_DEBUG, "Changing host key method to \"%s\"", methods_buffer);
-		return strdup(methods_buffer);
-	} else {
-		SSH_LOG(SSH_LOG_DEBUG, "No supported kex method for existing key in known_hosts file");
-		return NULL;
-	}
+    for (i = 0; preferred_hostkeys[i] != NULL; ++i) {
+        for (it = ssh_list_get_iterator(algo_list);
+             it != NULL;
+             it = ssh_list_get_iterator(algo_list)) {
+            const char *algo = ssh_iterator_value(const char *, it);
+            int cmp;
+            int ok;
 
+            cmp = strcmp(preferred_hostkeys[i], algo);
+            if (cmp == 0) {
+                ok = ssh_verify_existing_algo(SSH_HOSTKEYS, algo);
+                if (ok) {
+                    if (needcomma) {
+                        strncat(methods_buffer,
+                                ",",
+                                sizeof(methods_buffer) - strlen(methods_buffer) - 1);
+                    }
+                    strncat(methods_buffer,
+                            algo,
+                            sizeof(methods_buffer) - strlen(methods_buffer) - 1);
+                    needcomma = 1;
+                }
+            }
+
+            ssh_list_remove(algo_list, it);
+        }
+    }
+    ssh_list_free(algo_list);
+
+    if (strlen(methods_buffer) == 0) {
+        SSH_LOG(SSH_LOG_DEBUG,
+                "No supported kex method for existing key in known_hosts file");
+        return NULL;
+    }
+
+    SSH_LOG(SSH_LOG_DEBUG,
+            "Changing host key method to \"%s\"",
+            methods_buffer);
+
+    return strdup(methods_buffer);
 }
+
 /**
  * @brief sets the key exchange parameters to be sent to the server,
  *        in function of the options and available methods.
  */
-int set_client_kex(ssh_session session){
+int ssh_set_client_kex(ssh_session session){
     struct ssh_kex_struct *client= &session->next_crypto->client_kex;
     const char *wanted;
+    int ok;
     int i;
 
-    ssh_get_random(client->cookie, 16, 0);
+    ok = ssh_get_random(client->cookie, 16, 0);
+    if (!ok) {
+        ssh_set_error(session, SSH_FATAL, "PRNG error");
+        return SSH_ERROR;
+    }
 
     memset(client->methods, 0, KEX_METHODS_SIZE * sizeof(char **));
     /* first check if we have specific host key methods */
@@ -559,6 +667,10 @@ int set_client_kex(ssh_session session){
         if (wanted == NULL)
             wanted = default_methods[i];
         client->methods[i] = strdup(wanted);
+        if (client->methods[i] == NULL) {
+            ssh_set_error_oom(session);
+            return SSH_ERROR;
+        }
     }
 
     return SSH_OK;
@@ -589,10 +701,27 @@ int ssh_kex_select_methods (ssh_session session){
       session->next_crypto->kex_type=SSH_KEX_DH_GROUP14_SHA1;
     } else if(strcmp(session->next_crypto->kex_methods[SSH_KEX], "ecdh-sha2-nistp256") == 0){
       session->next_crypto->kex_type=SSH_KEX_ECDH_SHA2_NISTP256;
+    } else if(strcmp(session->next_crypto->kex_methods[SSH_KEX], "ecdh-sha2-nistp384") == 0){
+      session->next_crypto->kex_type=SSH_KEX_ECDH_SHA2_NISTP384;
+    } else if(strcmp(session->next_crypto->kex_methods[SSH_KEX], "ecdh-sha2-nistp521") == 0){
+      session->next_crypto->kex_type=SSH_KEX_ECDH_SHA2_NISTP521;
     } else if(strcmp(session->next_crypto->kex_methods[SSH_KEX], "curve25519-sha256@libssh.org") == 0){
       session->next_crypto->kex_type=SSH_KEX_CURVE25519_SHA256_LIBSSH_ORG;
+    } else if(strcmp(session->next_crypto->kex_methods[SSH_KEX], "curve25519-sha256") == 0){
+      session->next_crypto->kex_type=SSH_KEX_CURVE25519_SHA256;
     }
-
+    SSH_LOG(SSH_LOG_INFO, "Negotiated %s,%s,%s,%s,%s,%s,%s,%s,%s,%s",
+            session->next_crypto->kex_methods[SSH_KEX],
+            session->next_crypto->kex_methods[SSH_HOSTKEYS],
+            session->next_crypto->kex_methods[SSH_CRYPT_C_S],
+            session->next_crypto->kex_methods[SSH_CRYPT_S_C],
+            session->next_crypto->kex_methods[SSH_MAC_C_S],
+            session->next_crypto->kex_methods[SSH_MAC_S_C],
+            session->next_crypto->kex_methods[SSH_COMP_C_S],
+            session->next_crypto->kex_methods[SSH_COMP_S_C],
+            session->next_crypto->kex_methods[SSH_LANG_C_S],
+            session->next_crypto->kex_methods[SSH_LANG_S_C]
+    );
     return SSH_OK;
 }
 
@@ -612,7 +741,7 @@ int ssh_send_kex(ssh_session session, int server_kex) {
                        kex->cookie); /* cookie */
   if (rc != SSH_OK)
     goto error;
-  if (hashbufout_add_cookie(session) < 0) {
+  if (ssh_hashbufout_add_cookie(session) < 0) {
     goto error;
   }
 
@@ -624,10 +753,10 @@ int ssh_send_kex(ssh_session session, int server_kex) {
       goto error;
     }
 
-    if (buffer_add_ssh_string(session->out_hashbuf, str) < 0) {
+    if (ssh_buffer_add_ssh_string(session->out_hashbuf, str) < 0) {
       goto error;
     }
-    if (buffer_add_ssh_string(session->out_buffer, str) < 0) {
+    if (ssh_buffer_add_ssh_string(session->out_buffer, str) < 0) {
       goto error;
     }
     ssh_string_free(str);
@@ -642,7 +771,7 @@ int ssh_send_kex(ssh_session session, int server_kex) {
     goto error;
   }
 
-  if (packet_send(session) == SSH_ERROR) {
+  if (ssh_packet_send(session) == SSH_ERROR) {
     return -1;
   }
 
@@ -656,10 +785,14 @@ error:
 }
 
 /* returns 1 if at least one of the name algos is in the default algorithms table */
-int verify_existing_algo(int algo, const char *name){
+int ssh_verify_existing_algo(enum ssh_kex_types_e algo, const char *name)
+{
     char *ptr;
-    if(algo>9 || algo <0)
+
+    if (algo > SSH_LANG_S_C) {
         return -1;
+    }
+
     ptr=ssh_find_matching(supported_methods[algo],name);
     if(ptr){
         free(ptr);
@@ -668,4 +801,13 @@ int verify_existing_algo(int algo, const char *name){
     return 0;
 }
 
-/* vim: set ts=2 sw=2 et cindent: */
+/* returns a copy of the provided list if everything is supported,
+ * otherwise a new list of the supported algorithms */
+char *ssh_keep_known_algos(enum ssh_kex_types_e algo, const char *list)
+{
+    if (algo > SSH_LANG_S_C) {
+        return NULL;
+    }
+
+    return ssh_find_all_matching(supported_methods[algo], list);
+}
