@@ -2,6 +2,7 @@
  * This file is part of the SSH Library
  *
  * Copyright (c) 2009 by Aris Adamantiadis
+ * Copyright (C) 2016 g10 Code GmbH
  *
  * The SSH Library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,6 +20,8 @@
  * MA 02111-1307, USA.
  */
 
+#include "config.h"
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -27,6 +30,7 @@
 #include "libssh/session.h"
 #include "libssh/crypto.h"
 #include "libssh/wrapper.h"
+#include "libssh/string.h"
 
 #ifdef HAVE_LIBGCRYPT
 #include <gcrypt.h>
@@ -36,6 +40,8 @@ struct ssh_mac_ctx_struct {
   gcry_md_hd_t ctx;
 };
 
+static int libgcrypt_initialized = 0;
+
 static int alloc_key(struct ssh_cipher_struct *cipher) {
     cipher->key = malloc(cipher->keylen);
     if (cipher->key == NULL) {
@@ -46,7 +52,18 @@ static int alloc_key(struct ssh_cipher_struct *cipher) {
 }
 
 void ssh_reseed(void){
-	}
+}
+
+int ssh_get_random(void *where, int len, int strong)
+{
+    /* variable not used in gcrypt */
+    (void) strong;
+
+    /* not using GCRY_VERY_STRONG_RANDOM which is a bit overkill */
+    gcry_randomize(where,len,GCRY_STRONG_RANDOM);
+
+    return 1;
+}
 
 SHACTX sha1_init(void) {
   SHACTX ctx = NULL;
@@ -69,6 +86,59 @@ void sha1(unsigned char *digest, int len, unsigned char *hash) {
   gcry_md_hash_buffer(GCRY_MD_SHA1, hash, digest, len);
 }
 
+#ifdef HAVE_GCRYPT_ECC
+static int nid_to_md_algo(int nid)
+{
+    switch (nid) {
+    case NID_gcrypt_nistp256:
+        return GCRY_MD_SHA256;
+    case NID_gcrypt_nistp384:
+        return GCRY_MD_SHA384;
+    case NID_gcrypt_nistp521:
+        return GCRY_MD_SHA512;
+    }
+    return GCRY_MD_NONE;
+}
+
+void evp(int nid, unsigned char *digest, int len,
+         unsigned char *hash, unsigned int *hlen)
+{
+    int algo = nid_to_md_algo(nid);
+
+    /* Note: What gcrypt calls 'hash' is called 'digest' here and
+       vice-versa.  */
+    gcry_md_hash_buffer(algo, hash, digest, len);
+    *hlen = gcry_md_get_algo_dlen(algo);
+}
+
+EVPCTX evp_init(int nid)
+{
+    gcry_error_t err;
+    int algo = nid_to_md_algo(nid);
+    EVPCTX ctx;
+
+    err = gcry_md_open(&ctx, algo, 0);
+    if (err) {
+        return NULL;
+    }
+
+    return ctx;
+}
+
+void evp_update(EVPCTX ctx, const void *data, unsigned long len)
+{
+    gcry_md_write(ctx, data, len);
+}
+
+void evp_final(EVPCTX ctx, unsigned char *md, unsigned int *mdlen)
+{
+    int algo = gcry_md_get_algo(ctx);
+    *mdlen = gcry_md_get_algo_dlen(algo);
+    memcpy(md, gcry_md_read(ctx, algo), *mdlen);
+    gcry_md_close(ctx);
+}
+#endif
+
 SHA256CTX sha256_init(void) {
   SHA256CTX ctx = NULL;
   gcry_md_open(&ctx, GCRY_MD_SHA256, 0);
@@ -337,28 +407,6 @@ static void aes_decrypt(struct ssh_cipher_struct *cipher, void *in, void *out,
   gcry_cipher_decrypt(cipher->key[0], out, len, in, len);
 }
 
-static int des1_set_key(struct ssh_cipher_struct *cipher, void *key, void *IV){
-  if(!cipher->key){
-    if (alloc_key(cipher) < 0) {
-      return -1;
-    }
-    if (gcry_cipher_open(&cipher->key[0], GCRY_CIPHER_DES,
-          GCRY_CIPHER_MODE_CBC, 0)) {
-      SAFE_FREE(cipher->key);
-      return -1;
-    }
-    if (gcry_cipher_setkey(cipher->key[0], key, 8)) {
-      SAFE_FREE(cipher->key);
-      return -1;
-    }
-    if (gcry_cipher_setiv(cipher->key[0], IV, 8)) {
-      SAFE_FREE(cipher->key);
-      return -1;
-    }
-  }
-  return 0;
-}
-
 static int des3_set_key(struct ssh_cipher_struct *cipher, void *key, void *IV) {
   if (cipher->key == NULL) {
     if (alloc_key(cipher) < 0) {
@@ -382,17 +430,6 @@ static int des3_set_key(struct ssh_cipher_struct *cipher, void *key, void *IV) {
   return 0;
 }
 
-
-static void des1_1_encrypt(struct ssh_cipher_struct *cipher, void *in,
-    void *out, unsigned long len) {
-  gcry_cipher_encrypt(cipher->key[0], out, len, in, len);
-}
-
-static void des1_1_decrypt(struct ssh_cipher_struct *cipher, void *in,
-    void *out, unsigned long len) {
-  gcry_cipher_decrypt(cipher->key[0], out, len, in, len);
-}
-
 static void des3_encrypt(struct ssh_cipher_struct *cipher, void *in,
     void *out, unsigned long len) {
   gcry_cipher_encrypt(cipher->key[0], out, len, in, len);
@@ -403,71 +440,6 @@ static void des3_decrypt(struct ssh_cipher_struct *cipher, void *in,
   gcry_cipher_decrypt(cipher->key[0], out, len, in, len);
 }
 
-static int des3_1_set_key(struct ssh_cipher_struct *cipher, void *key, void *IV) {
-  if (cipher->key == NULL) {
-    if (alloc_key(cipher) < 0) {
-      return -1;
-    }
-    if (gcry_cipher_open(&cipher->key[0], GCRY_CIPHER_DES,
-          GCRY_CIPHER_MODE_CBC, 0)) {
-      SAFE_FREE(cipher->key);
-      return -1;
-    }
-    if (gcry_cipher_setkey(cipher->key[0], key, 8)) {
-      SAFE_FREE(cipher->key);
-      return -1;
-    }
-    if (gcry_cipher_setiv(cipher->key[0], IV, 8)) {
-      SAFE_FREE(cipher->key);
-      return -1;
-    }
-
-    if (gcry_cipher_open(&cipher->key[1], GCRY_CIPHER_DES,
-          GCRY_CIPHER_MODE_CBC, 0)) {
-      SAFE_FREE(cipher->key);
-      return -1;
-    }
-    if (gcry_cipher_setkey(cipher->key[1], (unsigned char *)key + 8, 8)) {
-      SAFE_FREE(cipher->key);
-      return -1;
-    }
-    if (gcry_cipher_setiv(cipher->key[1], (unsigned char *)IV + 8, 8)) {
-      SAFE_FREE(cipher->key);
-      return -1;
-    }
-
-    if (gcry_cipher_open(&cipher->key[2], GCRY_CIPHER_DES,
-          GCRY_CIPHER_MODE_CBC, 0)) {
-      SAFE_FREE(cipher->key);
-      return -1;
-    }
-    if (gcry_cipher_setkey(cipher->key[2], (unsigned char *)key + 16, 8)) {
-      SAFE_FREE(cipher->key);
-      return -1;
-    }
-    if (gcry_cipher_setiv(cipher->key[2], (unsigned char *)IV + 16, 8)) {
-      SAFE_FREE(cipher->key);
-      return -1;
-    }
-  }
-
-  return 0;
-}
-
-static void des3_1_encrypt(struct ssh_cipher_struct *cipher, void *in,
-    void *out, unsigned long len) {
-  gcry_cipher_encrypt(cipher->key[0], out, len, in, len);
-  gcry_cipher_decrypt(cipher->key[1], in, len, out, len);
-  gcry_cipher_encrypt(cipher->key[2], out, len, in, len);
-}
-
-static void des3_1_decrypt(struct ssh_cipher_struct *cipher, void *in,
-    void *out, unsigned long len) {
-  gcry_cipher_decrypt(cipher->key[2], out, len, in, len);
-  gcry_cipher_encrypt(cipher->key[1], in, len, out, len);
-  gcry_cipher_decrypt(cipher->key[0], out, len, in, len);
-}
-
 /* the table of supported ciphers */
 static struct ssh_cipher_struct ssh_ciphertab[] = {
   {
@@ -559,26 +531,7 @@ static struct ssh_cipher_struct ssh_ciphertab[] = {
     .decrypt     = des3_decrypt
   },
   {
-    .name            = "3des-cbc-ssh1",
-    .blocksize       = 8,
-    .keylen          = sizeof(gcry_cipher_hd_t) * 3,
-    .key             = NULL,
-    .keysize         = 192,
-    .set_encrypt_key = des3_1_set_key,
-    .set_decrypt_key = des3_1_set_key,
-    .encrypt     = des3_1_encrypt,
-    .decrypt     = des3_1_decrypt
-  },
-  {
-    .name            = "des-cbc-ssh1",
-    .blocksize       = 8,
-    .keylen          = sizeof(gcry_cipher_hd_t),
-    .key             = NULL,
-    .keysize         = 64,
-    .set_encrypt_key = des1_set_key,
-    .set_decrypt_key = des1_set_key,
-    .encrypt     = des1_1_encrypt,
-    .decrypt     = des1_1_decrypt
+    .name = "chacha20-poly1305@openssh.com"
   },
   {
     .name            = NULL,
@@ -598,4 +551,113 @@ struct ssh_cipher_struct *ssh_get_ciphertab(void)
   return ssh_ciphertab;
 }
 
+/*
+ * Extract an MPI from the given s-expression SEXP named NAME which is
+ * encoded using INFORMAT and store it in a newly allocated ssh_string
+ * encoded using OUTFORMAT.
+ */
+ssh_string ssh_sexp_extract_mpi(const gcry_sexp_t sexp,
+                                const char *name,
+                                enum gcry_mpi_format informat,
+                                enum gcry_mpi_format outformat)
+{
+    gpg_error_t err;
+    ssh_string result = NULL;
+    gcry_sexp_t fragment = NULL;
+    gcry_mpi_t mpi = NULL;
+    size_t size;
+
+    fragment = gcry_sexp_find_token(sexp, name, 0);
+    if (fragment == NULL) {
+        goto fail;
+    }
+
+    mpi = gcry_sexp_nth_mpi(fragment, 1, informat);
+    if (mpi == NULL) {
+        goto fail;
+    }
+
+    err = gcry_mpi_print(outformat, NULL, 0, &size, mpi);
+    if (err != 0) {
+        goto fail;
+    }
+
+    result = ssh_string_new(size);
+    if (result == NULL) {
+        goto fail;
+    }
+
+    err = gcry_mpi_print(outformat, ssh_string_data(result), size, NULL, mpi);
+    if (err != 0) {
+        ssh_string_burn(result);
+        ssh_string_free(result);
+        result = NULL;
+        goto fail;
+    }
+
+fail:
+    gcry_sexp_release(fragment);
+    gcry_mpi_release(mpi);
+    return result;
+}
+
+
+/**
+ * @internal
+ *
+ * @brief Initialize libgcrypt's subsystem
+ */
+int ssh_crypto_init(void)
+{
+    size_t i;
+
+    if (libgcrypt_initialized) {
+        return SSH_OK;
+    }
+
+    gcry_check_version(NULL);
+
+    /* While the secure memory is not set up */
+    gcry_control (GCRYCTL_SUSPEND_SECMEM_WARN);
+
+    if (!gcry_control(GCRYCTL_INITIALIZATION_FINISHED_P, 0)) {
+        gcry_control(GCRYCTL_INIT_SECMEM, 4096);
+        gcry_control(GCRYCTL_INITIALIZATION_FINISHED, 0);
+    }
+
+    /* Re-enable warning */
+    gcry_control (GCRYCTL_RESUME_SECMEM_WARN);
+
+    for (i = 0; ssh_ciphertab[i].name != NULL; i++) {
+        int cmp;
+        cmp = strcmp(ssh_ciphertab[i].name, "chacha20-poly1305@openssh.com");
+        if (cmp == 0) {
+            memcpy(&ssh_ciphertab[i],
+                   ssh_get_chacha20poly1305_cipher(),
+                   sizeof(struct ssh_cipher_struct));
+            break;
+        }
+    }
+
+    libgcrypt_initialized = 1;
+
+    return SSH_OK;
+}
+
+/**
+ * @internal
+ *
+ * @brief Finalize libgcrypt's subsystem
+ */
+void ssh_crypto_finalize(void)
+{
+    if (!libgcrypt_initialized) {
+        return;
+    }
+
+    gcry_control(GCRYCTL_TERM_SECMEM);
+
+    libgcrypt_initialized = 0;
+}
+
 #endif
