@@ -81,12 +81,12 @@ static ssh_message ssh_message_new(ssh_session session){
 static int ssh_message_reply_default(ssh_message msg) {
   SSH_LOG(SSH_LOG_FUNCTIONS, "Reporting unknown packet");
 
-  if (buffer_add_u8(msg->session->out_buffer, SSH2_MSG_UNIMPLEMENTED) < 0)
+  if (ssh_buffer_add_u8(msg->session->out_buffer, SSH2_MSG_UNIMPLEMENTED) < 0)
     goto error;
-  if (buffer_add_u32(msg->session->out_buffer,
+  if (ssh_buffer_add_u32(msg->session->out_buffer,
       htonl(msg->session->recv_seq-1)) < 0)
     goto error;
-  return packet_send(msg->session);
+  return ssh_packet_send(msg->session);
   error:
   return SSH_ERROR;
 }
@@ -166,90 +166,121 @@ static int ssh_execute_server_request(ssh_session session, ssh_message msg)
         case SSH_REQUEST_CHANNEL:
             channel = msg->channel_request.channel;
 
-            if (msg->channel_request.type == SSH_CHANNEL_REQUEST_PTY &&
-                ssh_callbacks_exists(channel->callbacks, channel_pty_request_function)) {
-                rc = channel->callbacks->channel_pty_request_function(session, channel,
-                        msg->channel_request.TERM,
-                        msg->channel_request.width, msg->channel_request.height,
-                        msg->channel_request.pxwidth, msg->channel_request.pxheight,
-                        channel->callbacks->userdata);
-                if (rc == 0) {
-                    ssh_message_channel_request_reply_success(msg);
-                } else {
-                    ssh_message_reply_default(msg);
+            if (msg->channel_request.type == SSH_CHANNEL_REQUEST_PTY){
+                ssh_callbacks_iterate(channel->callbacks,
+                                      ssh_channel_callbacks,
+                                      channel_pty_request_function) {
+                    rc = ssh_callbacks_iterate_exec(channel_pty_request_function,
+                                                    session,
+                                                    channel,
+                                                    msg->channel_request.TERM,
+                                                    msg->channel_request.width,
+                                                    msg->channel_request.height,
+                                                    msg->channel_request.pxwidth,
+                                                    msg->channel_request.pxheight);
+                    if (rc == 0) {
+                        ssh_message_channel_request_reply_success(msg);
+                    } else {
+                        ssh_message_reply_default(msg);
+                    }
+                    return SSH_OK;
                 }
-
-                return SSH_OK;
-            } else if (msg->channel_request.type == SSH_CHANNEL_REQUEST_SHELL &&
-                      ssh_callbacks_exists(channel->callbacks, channel_shell_request_function)) {
-                rc = channel->callbacks->channel_shell_request_function(session,
-                                                                        channel,
-                                                                        channel->callbacks->userdata);
-                if (rc == 0) {
-                    ssh_message_channel_request_reply_success(msg);
-                } else {
-                    ssh_message_reply_default(msg);
+                ssh_callbacks_iterate_end();
+            } else if (msg->channel_request.type == SSH_CHANNEL_REQUEST_SHELL){
+                ssh_callbacks_iterate(channel->callbacks,
+                                      ssh_channel_callbacks,
+                                      channel_shell_request_function) {
+                    rc = ssh_callbacks_iterate_exec(channel_shell_request_function,
+                                                    session,
+                                                    channel);
+                    if (rc == 0) {
+                        ssh_message_channel_request_reply_success(msg);
+                    } else {
+                        ssh_message_reply_default(msg);
+                    }
+                    return SSH_OK;
                 }
-
-                return SSH_OK;
-            } else if (msg->channel_request.type == SSH_CHANNEL_REQUEST_X11 &&
-                      ssh_callbacks_exists(channel->callbacks, channel_x11_req_function)) {
-                channel->callbacks->channel_x11_req_function(session,
-                                                             channel,
-                                                             msg->channel_request.x11_single_connection,
-                                                             msg->channel_request.x11_auth_protocol,
-                                                             msg->channel_request.x11_auth_cookie,
-                                                             msg->channel_request.x11_screen_number,
-                                                             channel->callbacks->userdata);
-                ssh_message_channel_request_reply_success(msg);
-
-                return SSH_OK;
-            } else if (msg->channel_request.type == SSH_CHANNEL_REQUEST_WINDOW_CHANGE &&
-                       ssh_callbacks_exists(channel->callbacks, channel_pty_window_change_function)) {
-                rc = channel->callbacks->channel_pty_window_change_function(session,
-                                                                            channel,
-                                                                            msg->channel_request.width, msg->channel_request.height,
-                                                                            msg->channel_request.pxwidth, msg->channel_request.pxheight,
-                                                                            channel->callbacks->userdata);
-            } else if (msg->channel_request.type == SSH_CHANNEL_REQUEST_EXEC &&
-                       ssh_callbacks_exists(channel->callbacks, channel_exec_request_function)) {
-                rc = channel->callbacks->channel_exec_request_function(session,
-                                                                       channel,
-                                                                       msg->channel_request.command,
-                                                                       channel->callbacks->userdata);
-                if (rc == 0) {
+                ssh_callbacks_iterate_end();
+            } else if (msg->channel_request.type == SSH_CHANNEL_REQUEST_X11){
+                ssh_callbacks_iterate(channel->callbacks,
+                                      ssh_channel_callbacks,
+                                      channel_x11_req_function) {
+                    ssh_callbacks_iterate_exec(channel_x11_req_function,
+                                               session,
+                                               channel,
+                                               msg->channel_request.x11_single_connection,
+                                               msg->channel_request.x11_auth_protocol,
+                                               msg->channel_request.x11_auth_cookie,
+                                               msg->channel_request.x11_screen_number);
                     ssh_message_channel_request_reply_success(msg);
-                } else {
-                    ssh_message_reply_default(msg);
+                    return SSH_OK;
                 }
-
-                return SSH_OK;
-            } else if (msg->channel_request.type == SSH_CHANNEL_REQUEST_ENV &&
-                       ssh_callbacks_exists(channel->callbacks, channel_env_request_function)) {
-                rc = channel->callbacks->channel_env_request_function(session,
-                                                                      channel,
-                                                                      msg->channel_request.var_name, msg->channel_request.var_value,
-                                                                      channel->callbacks->userdata);
-                if (rc == 0) {
-                    ssh_message_channel_request_reply_success(msg);
-                } else {
-                    ssh_message_reply_default(msg);
+                ssh_callbacks_iterate_end();
+            } else if (msg->channel_request.type == SSH_CHANNEL_REQUEST_WINDOW_CHANGE){
+                ssh_callbacks_iterate(channel->callbacks,
+                                      ssh_channel_callbacks,
+                                      channel_pty_window_change_function) {
+                    rc = ssh_callbacks_iterate_exec(channel_pty_window_change_function,
+                                                    session,
+                                                    channel,
+                                                    msg->channel_request.width,
+                                                    msg->channel_request.height,
+                                                    msg->channel_request.pxwidth,
+                                                    msg->channel_request.pxheight);
+                    return SSH_OK;
                 }
+                ssh_callbacks_iterate_end();
+            } else if (msg->channel_request.type == SSH_CHANNEL_REQUEST_EXEC){
+                ssh_callbacks_iterate(channel->callbacks,
+                                      ssh_channel_callbacks,
+                                      channel_exec_request_function) {
+                    rc = ssh_callbacks_iterate_exec(channel_exec_request_function,
+                                                    session,
+                                                    channel,
+                                                    msg->channel_request.command);
+                    if (rc == 0) {
+                        ssh_message_channel_request_reply_success(msg);
+                    } else {
+                        ssh_message_reply_default(msg);
+                    }
 
-                return SSH_OK;
-            } else if (msg->channel_request.type == SSH_CHANNEL_REQUEST_SUBSYSTEM &&
-                       ssh_callbacks_exists(channel->callbacks, channel_subsystem_request_function)) {
-                rc = channel->callbacks->channel_subsystem_request_function(session,
-                                                                            channel,
-                                                                            msg->channel_request.subsystem,
-                                                                            channel->callbacks->userdata);
-                if (rc == 0) {
-                    ssh_message_channel_request_reply_success(msg);
-                } else {
-                    ssh_message_reply_default(msg);
+                    return SSH_OK;
                 }
+                ssh_callbacks_iterate_end();
+            } else if (msg->channel_request.type == SSH_CHANNEL_REQUEST_ENV){
+                ssh_callbacks_iterate(channel->callbacks,
+                                      ssh_channel_callbacks,
+                                      channel_env_request_function) {
+                    rc = ssh_callbacks_iterate_exec(channel_env_request_function,
+                                                    session,
+                                                    channel,
+                                                    msg->channel_request.var_name,
+                                                    msg->channel_request.var_value);
+                    if (rc == 0) {
+                        ssh_message_channel_request_reply_success(msg);
+                    } else {
+                        ssh_message_reply_default(msg);
+                    }
+                    return SSH_OK;
+                }
+                ssh_callbacks_iterate_end();
+            } else if (msg->channel_request.type == SSH_CHANNEL_REQUEST_SUBSYSTEM){
+                ssh_callbacks_iterate(channel->callbacks,
+                                      ssh_channel_callbacks,
+                                      channel_subsystem_request_function) {
+                    rc = ssh_callbacks_iterate_exec(channel_subsystem_request_function,
+                                                    session,
+                                                    channel,
+                                                    msg->channel_request.subsystem);
+                    if (rc == 0) {
+                        ssh_message_channel_request_reply_success(msg);
+                    } else {
+                        ssh_message_reply_default(msg);
+                    }
 
-                return SSH_OK;
+                    return SSH_OK;
+                }
+                ssh_callbacks_iterate_end();
             }
             break;
         case SSH_REQUEST_SERVICE:
@@ -293,6 +324,21 @@ static int ssh_execute_client_request(ssh_session session, ssh_message msg)
             ssh_message_reply_default(msg);
         }
 
+        return SSH_OK;
+    } else if (msg->type == SSH_REQUEST_CHANNEL_OPEN
+               && msg->channel_request_open.type == SSH_CHANNEL_AUTH_AGENT
+               && ssh_callbacks_exists(session->common.callbacks, channel_open_request_auth_agent_function)) {
+        channel = session->common.callbacks->channel_open_request_auth_agent_function (session,
+                session->common.callbacks->userdata);
+
+        if (channel != NULL) {
+            rc = ssh_message_channel_request_open_reply_accept_channel(msg, channel);
+
+            return rc;
+        } else {
+            ssh_message_reply_default(msg);
+        }
+
         return SSH_OK;
     }
 
@@ -508,7 +554,8 @@ void ssh_message_free(ssh_message msg){
     case SSH_REQUEST_AUTH:
       SAFE_FREE(msg->auth_request.username);
       if (msg->auth_request.password) {
-        BURN_STRING(msg->auth_request.password);
+        explicit_bzero(msg->auth_request.password,
+                       strlen(msg->auth_request.password));
         SAFE_FREE(msg->auth_request.password);
       }
       ssh_key_free(msg->auth_request.pubkey);
@@ -564,7 +611,7 @@ SSH_PACKET_CALLBACK(ssh_packet_service_request){
 
   (void)type;
   (void)user;
-  service = buffer_get_ssh_string(packet);
+  service = ssh_buffer_get_ssh_string(packet);
   if (service == NULL) {
     ssh_set_error(session, SSH_FATAL, "Invalid SSH_MSG_SERVICE_REQUEST packet");
     goto error;
@@ -697,7 +744,7 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_request){
     ssh_string submethods = NULL;
 
     msg->auth_request.method = SSH_AUTH_METHOD_INTERACTIVE;
-    lang = buffer_get_ssh_string(packet);
+    lang = ssh_buffer_get_ssh_string(packet);
     if (lang == NULL) {
       goto error;
     }
@@ -707,7 +754,7 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_request){
      */
     ssh_string_free(lang);
 
-    submethods = buffer_get_ssh_string(packet);
+    submethods = ssh_buffer_get_ssh_string(packet);
     if (submethods == NULL) {
       goto error;
     }
@@ -753,7 +800,7 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_request){
         ssh_string sig_blob = NULL;
         ssh_buffer digest = NULL;
 
-        sig_blob = buffer_get_ssh_string(packet);
+        sig_blob = ssh_buffer_get_ssh_string(packet);
         if(sig_blob == NULL) {
             SSH_LOG(SSH_LOG_PACKET, "Invalid signature packet from peer");
             msg->auth_request.signature_state = SSH_PUBLICKEY_STATE_ERROR;
@@ -771,8 +818,8 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_request){
         rc = ssh_pki_signature_verify_blob(session,
                                            sig_blob,
                                            msg->auth_request.pubkey,
-                                           buffer_get_rest(digest),
-                                           buffer_get_rest_len(digest));
+                                           ssh_buffer_get(digest),
+                                           ssh_buffer_get_len(digest));
         ssh_string_free(sig_blob);
         ssh_buffer_free(digest);
         if (rc < 0) {
@@ -796,7 +843,7 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_request){
      ssh_string oid;
      char *hexa;
      int i;
-     buffer_get_u32(packet, &n_oid);
+     ssh_buffer_get_u32(packet, &n_oid);
      n_oid=ntohl(n_oid);
      if(n_oid > 100){
     	 ssh_set_error(session, SSH_FATAL, "USERAUTH_REQUEST: gssapi-with-mic OID count too big (%d)",n_oid);
@@ -809,7 +856,7 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_request){
     	 goto error;
      }
      for (i=0;i<(int) n_oid;++i){
-    	 oid=buffer_get_ssh_string(packet);
+    	 oid=ssh_buffer_get_ssh_string(packet);
     	 if(oid == NULL){
     		 for(i=i-1;i>=0;--i){
     			 SAFE_FREE(oids[i]);
@@ -923,11 +970,12 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_info_response){
 
       goto error;
     }
-  } else if (session->kbdint->nanswers > 0) {
+  } else if (session->kbdint->answers != NULL) {
       uint32_t n;
 
       for (n = 0; n < session->kbdint->nanswers; n++) {
-            BURN_STRING(session->kbdint->answers[n]);
+            explicit_bzero(session->kbdint->answers[n],
+                           strlen(session->kbdint->answers[n]));
             SAFE_FREE(session->kbdint->answers[n]);
       }
       SAFE_FREE(session->kbdint->answers);
@@ -952,7 +1000,7 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_info_response){
   }
   session->kbdint->nanswers = nanswers;
 
-  session->kbdint->answers = calloc(1, nanswers * sizeof(char *));
+  session->kbdint->answers = calloc(nanswers, sizeof(char *));
   if (session->kbdint->answers == NULL) {
     session->kbdint->nanswers = 0;
     ssh_set_error_oom(session);
@@ -963,7 +1011,7 @@ SSH_PACKET_CALLBACK(ssh_packet_userauth_info_response){
   }
 
   for (i = 0; i < nanswers; i++) {
-    tmp = buffer_get_ssh_string(packet);
+    tmp = ssh_buffer_get_ssh_string(packet);
     if (tmp == NULL) {
       ssh_set_error(session, SSH_FATAL, "Short INFO_RESPONSE packet");
       session->kbdint->nanswers = i;
@@ -1079,6 +1127,11 @@ SSH_PACKET_CALLBACK(ssh_packet_channel_open){
     goto end;
   }
 
+  if (strcmp(type_c,"auth-agent@openssh.com") == 0) {
+    msg->channel_request_open.type = SSH_CHANNEL_AUTH_AGENT;
+    goto end;
+  }
+
   msg->channel_request_open.type = SSH_CHANNEL_UNKNOWN;
   goto end;
 
@@ -1128,7 +1181,7 @@ int ssh_message_channel_request_open_reply_accept_channel(ssh_message msg, ssh_c
             "Accepting a channel request_open for chan %d",
             chan->remote_channel);
 
-    rc = packet_send(session);
+    rc = ssh_packet_send(session);
 
     return rc;
 }
@@ -1309,7 +1362,7 @@ int ssh_message_channel_request_reply_success(ssh_message msg) {
       return SSH_ERROR;
     }
 
-    return packet_send(msg->session);
+    return ssh_packet_send(msg->session);
   }
 
   SSH_LOG(SSH_LOG_PACKET,
@@ -1390,6 +1443,15 @@ SSH_PACKET_CALLBACK(ssh_packet_global_request){
             ssh_message_queue(session, msg);
             return rc;
         }
+    } else if(strcmp(request, "keepalive@openssh.com") == 0) {
+        msg->global_request.type = SSH_GLOBAL_REQUEST_KEEPALIVE;
+        msg->global_request.want_reply = want_reply;
+        SSH_LOG(SSH_LOG_PROTOCOL, "Received keepalive@openssh.com %d", want_reply);
+        if(ssh_callbacks_exists(session->common.callbacks, global_request_function)) {
+            session->common.callbacks->global_request_function(session, msg, session->common.callbacks->userdata);
+        } else {
+            ssh_message_global_request_reply_success(msg, 0);
+        }
     } else {
         SSH_LOG(SSH_LOG_PROTOCOL, "UNKNOWN SSH_MSG_GLOBAL_REQUEST %s %d", request, want_reply);
         rc = SSH_PACKET_NOT_USED;
@@ -1408,5 +1470,3 @@ error:
 #endif /* WITH_SERVER */
 
 /** @} */
-
-/* vim: set ts=4 sw=4 et cindent: */
