@@ -151,8 +151,6 @@ int ssh_options_copy(ssh_session src, ssh_session *dest) {
     new->opts.port                 = src->opts.port;
     new->opts.timeout              = src->opts.timeout;
     new->opts.timeout_usec         = src->opts.timeout_usec;
-    new->opts.ssh2                 = src->opts.ssh2;
-    new->opts.ssh1                 = src->opts.ssh1;
     new->opts.compressionlevel     = src->opts.compressionlevel;
     new->common.log_verbosity      = src->common.log_verbosity;
     new->common.callbacks          = src->common.callbacks;
@@ -162,23 +160,24 @@ int ssh_options_copy(ssh_session src, ssh_session *dest) {
     return 0;
 }
 
-int ssh_options_set_algo(ssh_session session, int algo,
-    const char *list) {
-  if (!verify_existing_algo(algo, list)) {
-    ssh_set_error(session, SSH_REQUEST_DENIED,
-        "Setting method: no algorithm for method \"%s\" (%s)\n",
-        ssh_kex_get_description(algo), list);
-    return -1;
-  }
+int ssh_options_set_algo(ssh_session session,
+                         enum ssh_kex_types_e algo,
+                         const char *list)
+{
+    char *p = NULL;
 
-  SAFE_FREE(session->opts.wanted_methods[algo]);
-  session->opts.wanted_methods[algo] = strdup(list);
-  if (session->opts.wanted_methods[algo] == NULL) {
-    ssh_set_error_oom(session);
-    return -1;
-  }
+    p = ssh_keep_known_algos(algo, list);
+    if (p == NULL) {
+        ssh_set_error(session, SSH_REQUEST_DENIED,
+                "Setting method: no algorithm for method \"%s\" (%s)",
+                ssh_kex_get_description(algo), list);
+        return -1;
+    }
 
-  return 0;
+    SAFE_FREE(session->opts.wanted_methods[algo]);
+    session->opts.wanted_methods[algo] = p;
+
+    return 0;
 }
 
 /**
@@ -234,15 +233,27 @@ int ssh_options_set_algo(ssh_session session, int algo,
  *                ~/.ssh/known_hosts.\n
  *                \n
  *                The known hosts file is used to certify remote hosts
- *                are genuine. It may include "%s" which will be
+ *                are genuine. It may include "%d" which will be
  *                replaced by the user home directory.
  *
- *              - SSH_OPTIONS_IDENTITY:
- *                Set the identity file name (const char *,format string).\n
+ *              - SSH_OPTIONS_GLOBAL_KNOWNHOSTS:
+ *                Set the global known hosts file name (const char *,format string).\n
+ *                \n
+ *                If the value is NULL, the directory is set to the
+ *                default global known hosts file, normally
+ *                /etc/ssh/ssh_known_hosts.\n
+ *                \n
+ *                The known hosts file is used to certify remote hosts
+ *                are genuine.
+ *
+ *              - SSH_OPTIONS_ADD_IDENTITY (or SSH_OPTIONS_IDENTITY):
+ *                Add a new identity file (const char *, format string) to
+ *                the identity list.\n
  *                \n
  *                By default identity, id_dsa and id_rsa are checked.\n
  *                \n
- *                The identity file used authenticate with public key.
+ *                The identity used to authenticate with public key will be
+ *                prepended to the list.
  *                It may include "%s" which will be replaced by the
  *                user home directory.
  *
@@ -254,12 +265,10 @@ int ssh_options_set_algo(ssh_session session, int algo,
  *                        (long).
  *
  *              - SSH_OPTIONS_SSH1:
- *                Allow or deny the connection to SSH1 servers
- *                (int, 0 is false).
+ *                Deprecated
  *
  *              - SSH_OPTIONS_SSH2:
- *                Allow or deny the connection to SSH2 servers
- *                (int, 0 is false).
+ *                Unused
  *
  *              - SSH_OPTIONS_LOG_VERBOSITY:
  *                Set the session logging verbosity (int).\n
@@ -373,6 +382,32 @@ int ssh_options_set_algo(ssh_session session, int algo,
  *                Set it to specify that GSSAPI should delegate credentials
  *                to the server (int, 0 = false).
  *
+ *              - SSH_OPTIONS_PASSWORD_AUTH
+ *                Set it if password authentication should be used
+ *                in ssh_userauth_auto_pubkey(). (int, 0=false).
+ *                Currently without effect (ssh_userauth_auto_pubkey doesn't use
+ *                password authentication).
+ *
+ *              - SSH_OPTIONS_PUBKEY_AUTH
+ *                Set it if pubkey authentication should be used
+ *                in ssh_userauth_auto_pubkey(). (int, 0=false).
+ *
+ *              - SSH_OPTIONS_KBDINT_AUTH
+ *                Set it if keyboard-interactive authentication should be used
+ *                in ssh_userauth_auto_pubkey(). (int, 0=false).
+ *                Currently without effect (ssh_userauth_auto_pubkey doesn't use
+ *                keyboard-interactive authentication).
+ *
+ *              - SSH_OPTIONS_GSSAPI_AUTH
+ *                Set it if gssapi authentication should be used
+ *                in ssh_userauth_auto_pubkey(). (int, 0=false).
+ *                Currently without effect (ssh_userauth_auto_pubkey doesn't use
+ *                gssapi authentication).
+ *
+ *              - SSH_OPTIONS_NODELAY
+ *                Set it to disable Nagle's Algorithm (TCP_NODELAY) on the
+ *                session socket. (int, 0=false)
+ *
  * @param  value The value to set. This is a generic pointer and the
  *               datatype which is used should be set according to the
  *               type set.
@@ -384,6 +419,7 @@ int ssh_options_set(ssh_session session, enum ssh_options_e type,
     const char *v;
     char *p, *q;
     long int i;
+    unsigned int u;
     int rc;
 
     if (session == NULL) {
@@ -573,6 +609,27 @@ int ssh_options_set(ssh_session session, enum ssh_options_e type,
                 }
             }
             break;
+        case SSH_OPTIONS_GLOBAL_KNOWNHOSTS:
+            v = value;
+            SAFE_FREE(session->opts.global_knownhosts);
+            if (v == NULL) {
+                session->opts.global_knownhosts =
+                    strdup("/etc/ssh/ssh_known_hosts");
+                if (session->opts.global_knownhosts == NULL) {
+                    ssh_set_error_oom(session);
+                    return -1;
+                }
+            } else if (v[0] == '\0') {
+                ssh_set_error_invalid(session);
+                return -1;
+            } else {
+                session->opts.global_knownhosts = strdup(v);
+                if (session->opts.global_knownhosts == NULL) {
+                    ssh_set_error_oom(session);
+                    return -1;
+                }
+            }
+            break;
         case SSH_OPTIONS_TIMEOUT:
             if (value == NULL) {
                 ssh_set_error_invalid(session);
@@ -602,32 +659,8 @@ int ssh_options_set(ssh_session session, enum ssh_options_e type,
             }
             break;
         case SSH_OPTIONS_SSH1:
-            if (value == NULL) {
-                ssh_set_error_invalid(session);
-                return -1;
-            } else {
-                int *x = (int *) value;
-                if (*x < 0) {
-                    ssh_set_error_invalid(session);
-                    return -1;
-                }
-
-                session->opts.ssh1 = *x;
-            }
             break;
         case SSH_OPTIONS_SSH2:
-            if (value == NULL) {
-                ssh_set_error_invalid(session);
-                return -1;
-            } else {
-                int *x = (int *) value;
-                if (*x < 0) {
-                    ssh_set_error_invalid(session);
-                    return -1;
-                }
-
-                session->opts.ssh2 = *x & 0xffff;
-            }
             break;
         case SSH_OPTIONS_LOG_VERBOSITY:
             if (value == NULL) {
@@ -857,7 +890,39 @@ int ssh_options_set(ssh_session session, enum ssh_options_e type,
                 session->opts.gss_delegate_creds = (x & 0xff);
             }
             break;
-
+        case SSH_OPTIONS_PASSWORD_AUTH:
+        case SSH_OPTIONS_PUBKEY_AUTH:
+        case SSH_OPTIONS_KBDINT_AUTH:
+        case SSH_OPTIONS_GSSAPI_AUTH:
+            u = 0;
+            if (value == NULL) {
+                ssh_set_error_invalid(session);
+                return -1;
+            } else {
+                int x = *(int *)value;
+                u = type == SSH_OPTIONS_PASSWORD_AUTH ?
+                    SSH_OPT_FLAG_PASSWORD_AUTH:
+                    type == SSH_OPTIONS_PUBKEY_AUTH ?
+                        SSH_OPT_FLAG_PUBKEY_AUTH:
+                        type == SSH_OPTIONS_KBDINT_AUTH ?
+                            SSH_OPT_FLAG_KBDINT_AUTH:
+                            SSH_OPT_FLAG_GSSAPI_AUTH;
+                if (x != 0){
+                    session->opts.flags |= u;
+                } else {
+                    session->opts.flags &= ~u;
+                }
+            }
+            break;
+        case SSH_OPTIONS_NODELAY:
+            if (value == NULL) {
+                ssh_set_error_invalid(session);
+                return -1;
+            } else {
+                int *x = (int *) value;
+                session->opts.nodelay = (*x & 0xff) > 0 ? 1 : 0;
+            }
+            break;
         default:
             ssh_set_error(session, SSH_REQUEST_DENIED, "Unknown ssh option %d", type);
             return -1;
@@ -917,13 +982,9 @@ int ssh_options_get_port(ssh_session session, unsigned int* port_target) {
  *                ~/.ssh/config file.
  *
  *              - SSH_OPTIONS_IDENTITY:
- *                Set the identity file name (const char *,format string).\n
+ *                Get the first identity file name (const char *).\n
  *                \n
- *                By default identity, id_dsa and id_rsa are checked.\n
- *                \n
- *                The identity file used authenticate with public key.
- *                It may include "%s" which will be replaced by the
- *                user home directory.
+ *                By default identity, id_dsa and id_rsa are checked.
  *
  *              - SSH_OPTIONS_PROXYCOMMAND:
  *                Get the proxycommand necessary to log into the
@@ -1024,12 +1085,6 @@ int ssh_options_getopt(ssh_session session, int *argcptr, char **argv) {
   int compress = 0;
   int cont = 1;
   int current = 0;
-#ifdef WITH_SSH1
-  int ssh1 = 1;
-#else
-  int ssh1 = 0;
-#endif
-  int ssh2 = 1;
 #ifdef _MSC_VER
     /* Not supported with a Microsoft compiler */
     return -1;
@@ -1065,12 +1120,8 @@ int ssh_options_getopt(ssh_session session, int *argcptr, char **argv) {
         compress++;
         break;
       case '2':
-        ssh2 = 1;
-        ssh1 = 0;
         break;
       case '1':
-        ssh2 = 0;
-        ssh1 = 1;
         break;
       default:
         {
@@ -1168,10 +1219,9 @@ int ssh_options_getopt(ssh_session session, int *argcptr, char **argv) {
     }
   }
 
-  ssh_options_set(session, SSH_OPTIONS_PORT_STR, port);
-
-  ssh_options_set(session, SSH_OPTIONS_SSH1, &ssh1);
-  ssh_options_set(session, SSH_OPTIONS_SSH2, &ssh2);
+  if (port != NULL) {
+    ssh_options_set(session, SSH_OPTIONS_PORT_STR, port);
+  }
 
   if (!cont) {
     return SSH_ERROR;
@@ -1378,6 +1428,9 @@ static int ssh_bind_set_key(ssh_bind sshbind, char **key_loc,
  *                      - SSH_BIND_OPTIONS_BANNER:
  *                        Set the server banner sent to clients (const char *).
  *
+ *                      - SSH_BIND_OPTIONS_IMPORT_KEY:
+ *                        Set the Private Key for the server directly (ssh_key)
+ *
  * @param  value        The value to set. This is a generic pointer and the
  *                      datatype which should be used is described at the
  *                      corresponding value of type above.
@@ -1411,8 +1464,15 @@ int ssh_bind_options_set(ssh_bind sshbind, enum ssh_bind_options_e type,
           key_type = ssh_key_type(key);
           switch (key_type) {
           case SSH_KEYTYPE_DSS:
+#ifdef HAVE_DSA
               bind_key_loc = &sshbind->dsa;
               bind_key_path_loc = &sshbind->dsakey;
+#else
+              ssh_set_error(sshbind,
+                      SSH_FATAL,
+                      "DSS key used and libssh compiled "
+                      "without DSA support");
+#endif
               break;
           case SSH_KEYTYPE_ECDSA:
 #ifdef HAVE_ECC
@@ -1426,7 +1486,6 @@ int ssh_bind_options_set(ssh_bind sshbind, enum ssh_bind_options_e type,
 #endif
               break;
           case SSH_KEYTYPE_RSA:
-          case SSH_KEYTYPE_RSA1:
               bind_key_loc = &sshbind->rsa;
               bind_key_path_loc = &sshbind->rsakey;
               break;
@@ -1456,6 +1515,54 @@ int ssh_bind_options_set(ssh_bind sshbind, enum ssh_bind_options_e type,
           *bind_key_loc = key;
       }
       break;
+    case SSH_BIND_OPTIONS_IMPORT_KEY:
+        if (value == NULL) {
+            ssh_set_error_invalid(sshbind);
+            return -1;
+        } else {
+            int key_type;
+            ssh_key *bind_key_loc = NULL;
+            ssh_key key = (ssh_key)value;
+
+            key_type = ssh_key_type(key);
+            switch (key_type) {
+                case SSH_KEYTYPE_DSS:
+#ifdef HAVE_DSA
+                    bind_key_loc = &sshbind->dsa;
+#else
+                    ssh_set_error(sshbind,
+                                  SSH_FATAL,
+                                  "DSA key used and libssh compiled "
+                                  "without DSA support");
+#endif
+                    break;
+                case SSH_KEYTYPE_ECDSA:
+#ifdef HAVE_ECC
+                    bind_key_loc = &sshbind->ecdsa;
+#else
+                    ssh_set_error(sshbind,
+                                  SSH_FATAL,
+                                  "ECDSA key used and libssh compiled "
+                                  "without ECDSA support");
+#endif
+                    break;
+                case SSH_KEYTYPE_RSA:
+                    bind_key_loc = &sshbind->rsa;
+                    break;
+                case SSH_KEYTYPE_ED25519:
+                    bind_key_loc = &sshbind->ed25519;
+                    break;
+                default:
+                    ssh_set_error(sshbind,
+                                  SSH_FATAL,
+                                  "Unsupported key type %d", key_type);
+            }
+            if (bind_key_loc == NULL)
+                return -1;
+            ssh_key_free(*bind_key_loc);
+            *bind_key_loc = key;
+        }
+        break;
     case SSH_BIND_OPTIONS_BINDADDR:
       if (value == NULL) {
         ssh_set_error_invalid(sshbind);
@@ -1565,5 +1672,3 @@ int ssh_bind_options_set(ssh_bind sshbind, enum ssh_bind_options_e type,
 #endif
 
 /** @} */
-
-/* vim: set ts=4 sw=4 et cindent: */
