@@ -25,6 +25,8 @@
 #ifndef _PKI_CRYPTO_H
 #define _PKI_CRYPTO_H
 
+#include "config.h"
+
 #include "libssh/priv.h"
 
 #include <openssl/pem.h>
@@ -451,11 +453,30 @@ int pki_key_generate_rsa(ssh_key key, int parameter){
 
 int pki_key_generate_dss(ssh_key key, int parameter){
     int rc;
+#if OPENSSL_VERSION_NUMBER > 0x10100000L
+    key->dsa = DSA_new();
+    if (key->dsa == NULL) {
+        return SSH_ERROR;
+    }
+    rc = DSA_generate_parameters_ex(key->dsa,
+                                    parameter,
+                                    NULL,  /* seed */
+                                    0,     /* seed_len */
+                                    NULL,  /* counter_ret */
+                                    NULL,  /* h_ret */
+                                    NULL); /* cb */
+    if (rc != 1) {
+        DSA_free(key->dsa);
+        key->dsa = NULL;
+        return SSH_ERROR;
+    }
+#else
     key->dsa = DSA_generate_parameters(parameter, NULL, 0, NULL, NULL,
             NULL, NULL);
     if(key->dsa == NULL){
         return SSH_ERROR;
     }
+#endif
     rc = DSA_generate_key(key->dsa);
     if (rc != 1){
         DSA_free(key->dsa);
@@ -616,11 +637,6 @@ ssh_string pki_private_key_to_pem(const ssh_key key,
     BIO *mem;
     int rc;
 
-    /* needed for openssl initialization */
-    if (ssh_init() < 0) {
-        return NULL;
-    }
-
     mem = BIO_new(BIO_s_mem());
     if (mem == NULL) {
         return NULL;
@@ -641,7 +657,7 @@ ssh_string pki_private_key_to_pem(const ssh_key key,
             } else {
                 rc = PEM_write_bio_DSAPrivateKey(mem,
                                                  key->dsa,
-                                                 NULL, /* cipher */
+                                                 EVP_aes_128_cbc(),
                                                  NULL, /* kstr */
                                                  0, /* klen */
                                                  NULL, /* auth_fn */
@@ -666,7 +682,7 @@ ssh_string pki_private_key_to_pem(const ssh_key key,
             } else {
                 rc = PEM_write_bio_RSAPrivateKey(mem,
                                                  key->rsa,
-                                                 NULL, /* cipher */
+                                                 EVP_aes_128_cbc(),
                                                  NULL, /* kstr */
                                                  0, /* klen */
                                                  NULL, /* auth_fn */
@@ -676,8 +692,8 @@ ssh_string pki_private_key_to_pem(const ssh_key key,
                 goto err;
             }
             break;
-        case SSH_KEYTYPE_ECDSA:
 #ifdef HAVE_ECC
+        case SSH_KEYTYPE_ECDSA:
             if (passphrase == NULL) {
                 struct pem_get_password_struct pgp = { auth_fn, auth_data };
 
@@ -691,7 +707,7 @@ ssh_string pki_private_key_to_pem(const ssh_key key,
             } else {
                 rc = PEM_write_bio_ECPrivateKey(mem,
                                                 key->ecdsa,
-                                                NULL, /* cipher */
+                                                EVP_aes_128_cbc(),
                                                 NULL, /* kstr */
                                                 0, /* klen */
                                                 NULL, /* auth_fn */
@@ -704,11 +720,14 @@ ssh_string pki_private_key_to_pem(const ssh_key key,
 #endif
         case SSH_KEYTYPE_ED25519:
             BIO_free(mem);
-            ssh_pki_log("PEM output not supported for key type ssh-ed25519");
+            SSH_LOG(SSH_LOG_WARN, "PEM output not supported for key type ssh-ed25519");
             return NULL;
+        case SSH_KEYTYPE_DSS_CERT01:
+        case SSH_KEYTYPE_RSA_CERT01:
         case SSH_KEYTYPE_UNKNOWN:
+        default:
             BIO_free(mem);
-            ssh_pki_log("Unkown or invalid private key type %d", key->type);
+            SSH_LOG(SSH_LOG_WARN, "Unkown or invalid private key type %d", key->type);
             return NULL;
     }
 
@@ -744,14 +763,9 @@ ssh_key pki_private_key_from_base64(const char *b64_key,
     void *ecdsa = NULL;
 #endif
 
-    /* needed for openssl initialization */
-    if (ssh_init() < 0) {
-        return NULL;
-    }
-
     type = pki_privatekey_type_from_string(b64_key);
     if (type == SSH_KEYTYPE_UNKNOWN) {
-        ssh_pki_log("Unknown or invalid private key.");
+        SSH_LOG(SSH_LOG_WARN, "Unknown or invalid private key.");
         return NULL;
     }
 
@@ -775,8 +789,9 @@ ssh_key pki_private_key_from_base64(const char *b64_key,
             BIO_free(mem);
 
             if (dsa == NULL) {
-                ssh_pki_log("Parsing private key: %s",
-                            ERR_error_string(ERR_get_error(), NULL));
+                SSH_LOG(SSH_LOG_WARN,
+                        "Parsing private key: %s",
+                        ERR_error_string(ERR_get_error(), NULL));
                 return NULL;
             }
 
@@ -799,8 +814,9 @@ ssh_key pki_private_key_from_base64(const char *b64_key,
             BIO_free(mem);
 
             if (rsa == NULL) {
-                ssh_pki_log("Parsing private key: %s",
-                            ERR_error_string(ERR_get_error(),NULL));
+                SSH_LOG(SSH_LOG_WARN,
+                        "Parsing private key: %s",
+                        ERR_error_string(ERR_get_error(),NULL));
                 return NULL;
             }
 
@@ -823,8 +839,9 @@ ssh_key pki_private_key_from_base64(const char *b64_key,
             BIO_free(mem);
 
             if (ecdsa == NULL) {
-                ssh_pki_log("Parsing private key: %s",
-                            ERR_error_string(ERR_get_error(), NULL));
+                SSH_LOG(SSH_LOG_WARN,
+                        "Parsing private key: %s",
+                        ERR_error_string(ERR_get_error(), NULL));
                 return NULL;
             }
 
@@ -832,9 +849,11 @@ ssh_key pki_private_key_from_base64(const char *b64_key,
 #endif
         case SSH_KEYTYPE_ED25519:
             /* Cannot open ed25519 keys with libcrypto */
+        case SSH_KEYTYPE_DSS_CERT01:
+        case SSH_KEYTYPE_RSA_CERT01:
         case SSH_KEYTYPE_UNKNOWN:
             BIO_free(mem);
-            ssh_pki_log("Unkown or invalid private key type %d", type);
+            SSH_LOG(SSH_LOG_WARN, "Unkown or invalid private key type %d", type);
             return NULL;
     }
 
@@ -882,29 +901,29 @@ int pki_pubkey_build_dss(ssh_key key,
         return SSH_ERROR;
     }
 
-    bp = make_string_bn(p);
-    bq = make_string_bn(q);
-    bg = make_string_bn(g);
-    bpub_key = make_string_bn(pubkey);
+    bp = ssh_make_string_bn(p);
+    bq = ssh_make_string_bn(q);
+    bg = ssh_make_string_bn(g);
+    bpub_key = ssh_make_string_bn(pubkey);
     if (bp == NULL || bq == NULL ||
         bg == NULL || bpub_key == NULL) {
-        DSA_free(key->dsa);
-        return SSH_ERROR;
+        goto fail;
     }
 
     rc = DSA_set0_pqg(key->dsa, bp, bq, bg);
     if (rc == 0) {
-        DSA_free(key->dsa);
-        return SSH_ERROR;
+        goto fail;
     }
 
     rc = DSA_set0_key(key->dsa, bpub_key, NULL);
     if (rc == 0) {
-        DSA_free(key->dsa);
-        return SSH_ERROR;
+        goto fail;
     }
 
     return SSH_OK;
+fail:
+    DSA_free(key->dsa);
+    return SSH_ERROR;
 }
 
 int pki_pubkey_build_rsa(ssh_key key,
@@ -918,20 +937,21 @@ int pki_pubkey_build_rsa(ssh_key key,
         return SSH_ERROR;
     }
 
-    be = make_string_bn(e);
-    bn = make_string_bn(n);
+    be = ssh_make_string_bn(e);
+    bn = ssh_make_string_bn(n);
     if (be == NULL || bn == NULL) {
-        RSA_free(key->rsa);
-        return SSH_ERROR;
+        goto fail;
     }
 
     rc = RSA_set0_key(key->rsa, bn, be, NULL);
     if (rc == 0) {
-        RSA_free(key->rsa);
-        return SSH_ERROR;
+        goto fail;
     }
 
     return SSH_OK;
+fail:
+    RSA_free(key->rsa);
+    return SSH_ERROR;
 }
 
 ssh_string pki_publickey_to_blob(const ssh_key key)
@@ -951,13 +971,22 @@ ssh_string pki_publickey_to_blob(const ssh_key key)
         return NULL;
     }
 
+    if (key->cert != NULL) {
+        rc = ssh_buffer_add_buffer(buffer, key->cert);
+        if (rc < 0) {
+            ssh_buffer_free(buffer);
+            return NULL;
+        }
+        goto makestring;
+    }
+
     type_s = ssh_string_from_char(key->type_c);
     if (type_s == NULL) {
         ssh_buffer_free(buffer);
         return NULL;
     }
 
-    rc = buffer_add_ssh_string(buffer, type_s);
+    rc = ssh_buffer_add_ssh_string(buffer, type_s);
     ssh_string_free(type_s);
     if (rc < 0) {
         ssh_buffer_free(buffer);
@@ -968,37 +997,37 @@ ssh_string pki_publickey_to_blob(const ssh_key key)
         case SSH_KEYTYPE_DSS: {
             const BIGNUM *bp, *bq, *bg, *bpub_key;
             DSA_get0_pqg(key->dsa, &bp, &bq, &bg);
-            p = make_bignum_string((BIGNUM *)bp);
+            p = ssh_make_bignum_string((BIGNUM *)bp);
             if (p == NULL) {
                 goto fail;
             }
 
-            q = make_bignum_string((BIGNUM *)bq);
+            q = ssh_make_bignum_string((BIGNUM *)bq);
             if (q == NULL) {
                 goto fail;
             }
 
-            g = make_bignum_string((BIGNUM *)bg);
+            g = ssh_make_bignum_string((BIGNUM *)bg);
             if (g == NULL) {
                 goto fail;
             }
 
             DSA_get0_key(key->dsa, &bpub_key, NULL);
-            n = make_bignum_string((BIGNUM *)bpub_key);
+            n = ssh_make_bignum_string((BIGNUM *)bpub_key);
             if (n == NULL) {
                 goto fail;
             }
 
-            if (buffer_add_ssh_string(buffer, p) < 0) {
+            if (ssh_buffer_add_ssh_string(buffer, p) < 0) {
                 goto fail;
             }
-            if (buffer_add_ssh_string(buffer, q) < 0) {
+            if (ssh_buffer_add_ssh_string(buffer, q) < 0) {
                 goto fail;
             }
-            if (buffer_add_ssh_string(buffer, g) < 0) {
+            if (ssh_buffer_add_ssh_string(buffer, g) < 0) {
                 goto fail;
             }
-            if (buffer_add_ssh_string(buffer, n) < 0) {
+            if (ssh_buffer_add_ssh_string(buffer, n) < 0) {
                 goto fail;
             }
 
@@ -1021,20 +1050,20 @@ ssh_string pki_publickey_to_blob(const ssh_key key)
         case SSH_KEYTYPE_RSA1: {
             const BIGNUM *be, *bn;
             RSA_get0_key(key->rsa, &bn, &be, NULL);
-            e = make_bignum_string((BIGNUM *)be);
+            e = ssh_make_bignum_string((BIGNUM *)be);
             if (e == NULL) {
                 goto fail;
             }
 
-            n = make_bignum_string((BIGNUM *)bn);
+            n = ssh_make_bignum_string((BIGNUM *)bn);
             if (n == NULL) {
                 goto fail;
             }
 
-            if (buffer_add_ssh_string(buffer, e) < 0) {
+            if (ssh_buffer_add_ssh_string(buffer, e) < 0) {
                 goto fail;
             }
-            if (buffer_add_ssh_string(buffer, n) < 0) {
+            if (ssh_buffer_add_ssh_string(buffer, n) < 0) {
                 goto fail;
             }
 
@@ -1061,7 +1090,7 @@ ssh_string pki_publickey_to_blob(const ssh_key key)
                 return NULL;
             }
 
-            rc = buffer_add_ssh_string(buffer, type_s);
+            rc = ssh_buffer_add_ssh_string(buffer, type_s);
             ssh_string_free(type_s);
             if (rc < 0) {
                 ssh_buffer_free(buffer);
@@ -1074,7 +1103,7 @@ ssh_string pki_publickey_to_blob(const ssh_key key)
                 return NULL;
             }
 
-            rc = buffer_add_ssh_string(buffer, type_s);
+            rc = ssh_buffer_add_ssh_string(buffer, type_s);
             ssh_string_free(type_s);
             if (rc < 0) {
                 ssh_buffer_free(buffer);
@@ -1088,7 +1117,7 @@ ssh_string pki_publickey_to_blob(const ssh_key key)
                 return NULL;
             }
 
-            rc = buffer_add_ssh_string(buffer, e);
+            rc = ssh_buffer_add_ssh_string(buffer, e);
             if (rc < 0) {
                 goto fail;
             }
@@ -1110,12 +1139,13 @@ ssh_string pki_publickey_to_blob(const ssh_key key)
             goto fail;
     }
 
-    str = ssh_string_new(buffer_get_rest_len(buffer));
+makestring:
+    str = ssh_string_new(ssh_buffer_get_len(buffer));
     if (str == NULL) {
         goto fail;
     }
 
-    rc = ssh_string_fill(str, buffer_get_rest(buffer), buffer_get_rest_len(buffer));
+    rc = ssh_string_fill(str, ssh_buffer_get(buffer), ssh_buffer_get_len(buffer));
     if (rc < 0) {
         goto fail;
     }
@@ -1140,37 +1170,6 @@ fail:
     return NULL;
 }
 
-int pki_export_pubkey_rsa1(const ssh_key key,
-                           const char *host,
-                           char *rsa1,
-                           size_t rsa1_len)
-{
-    char *e;
-    char *n;
-    int rsa_size = RSA_size(key->rsa);
-    const BIGNUM *be, *bn;
-
-    RSA_get0_key(key->rsa, &bn, &be, NULL);
-    e = bignum_bn2dec(be);
-    if (e == NULL) {
-        return SSH_ERROR;
-    }
-
-    n = bignum_bn2dec(bn);
-    if (n == NULL) {
-        OPENSSL_free(e);
-        return SSH_ERROR;
-    }
-
-    snprintf(rsa1, rsa1_len,
-             "%s %d %s %s\n",
-             host, rsa_size << 3, e, n);
-    OPENSSL_free(e);
-    OPENSSL_free(n);
-
-    return SSH_OK;
-}
-
 /**
  * @internal
  *
@@ -1230,12 +1229,12 @@ static ssh_string pki_dsa_signature_to_blob(const ssh_signature sig)
     int s_len, s_offset_in, s_offset_out;
 
     DSA_SIG_get0(sig->dsa_sig, &pr, &ps);
-    r = make_bignum_string((BIGNUM *)pr);
+    r = ssh_make_bignum_string((BIGNUM *)pr);
     if (r == NULL) {
         return NULL;
     }
 
-    s = make_bignum_string((BIGNUM *)ps);
+    s = ssh_make_bignum_string((BIGNUM *)ps);
     if (s == NULL) {
         ssh_string_free(r);
         return NULL;
@@ -1296,37 +1295,37 @@ ssh_string pki_signature_to_blob(const ssh_signature sig)
             }
 
             ECDSA_SIG_get0(sig->ecdsa_sig, &pr, &ps);
-            r = make_bignum_string((BIGNUM *)pr);
+            r = ssh_make_bignum_string((BIGNUM *)pr);
             if (r == NULL) {
                 ssh_buffer_free(b);
                 return NULL;
             }
-            rc = buffer_add_ssh_string(b, r);
+            rc = ssh_buffer_add_ssh_string(b, r);
             ssh_string_free(r);
             if (rc < 0) {
                 ssh_buffer_free(b);
                 return NULL;
             }
 
-            s = make_bignum_string((BIGNUM *)ps);
+            s = ssh_make_bignum_string((BIGNUM *)ps);
             if (s == NULL) {
                 ssh_buffer_free(b);
                 return NULL;
             }
-            rc = buffer_add_ssh_string(b, s);
+            rc = ssh_buffer_add_ssh_string(b, s);
             ssh_string_free(s);
             if (rc < 0) {
                 ssh_buffer_free(b);
                 return NULL;
             }
 
-            sig_blob = ssh_string_new(buffer_get_rest_len(b));
+            sig_blob = ssh_string_new(ssh_buffer_get_len(b));
             if (sig_blob == NULL) {
                 ssh_buffer_free(b);
                 return NULL;
             }
 
-            ssh_string_fill(sig_blob, buffer_get_rest(b), buffer_get_rest_len(b));
+            ssh_string_fill(sig_blob, ssh_buffer_get(b), ssh_buffer_get_len(b));
             ssh_buffer_free(b);
             break;
         }
@@ -1336,7 +1335,7 @@ ssh_string pki_signature_to_blob(const ssh_signature sig)
             break;
         default:
         case SSH_KEYTYPE_UNKNOWN:
-            ssh_pki_log("Unknown signature key type: %s", sig->type_c);
+            SSH_LOG(SSH_LOG_WARN, "Unknown signature key type: %s", sig->type_c);
             return NULL;
     }
 
@@ -1356,19 +1355,21 @@ static ssh_signature pki_signature_from_rsa_blob(const ssh_key pubkey,
     size_t len = ssh_string_len(sig_blob);
 
     if (pubkey->rsa == NULL) {
-        ssh_pki_log("Pubkey RSA field NULL");
+        SSH_LOG(SSH_LOG_WARN, "Pubkey RSA field NULL");
         goto errout;
     }
 
     rsalen = RSA_size(pubkey->rsa);
     if (len > rsalen) {
-        ssh_pki_log("Signature is too big: %lu > %lu",
-                    (unsigned long)len, (unsigned long)rsalen);
+        SSH_LOG(SSH_LOG_WARN,
+                "Signature is too big: %lu > %lu",
+                (unsigned long)len,
+                (unsigned long)rsalen);
         goto errout;
     }
 
 #ifdef DEBUG_CRYPTO
-    ssh_pki_log("RSA signature len: %lu", (unsigned long)len);
+    SSH_LOG(SSH_LOG_WARN, "RSA signature len: %lu", (unsigned long)len);
     ssh_print_hexa("RSA signature", ssh_string_data(sig_blob), len);
 #endif
 
@@ -1376,8 +1377,10 @@ static ssh_signature pki_signature_from_rsa_blob(const ssh_key pubkey,
         sig->rsa_sig = ssh_string_copy(sig_blob);
     } else {
         /* pad the blob to the expected rsalen size */
-        ssh_pki_log("RSA signature len %lu < %lu",
-                    (unsigned long)len, (unsigned long)rsalen);
+        SSH_LOG(SSH_LOG_DEBUG,
+                "RSA signature len %lu < %lu",
+                (unsigned long)len,
+                (unsigned long)rsalen);
 
         pad_len = rsalen - len;
 
@@ -1390,7 +1393,7 @@ static ssh_signature pki_signature_from_rsa_blob(const ssh_key pubkey,
         blob_orig = (char *) ssh_string_data(sig_blob);
 
         /* front-pad the buffer with zeroes */
-        BURN_BUFFER(blob_padded_data, pad_len);
+        explicit_bzero(blob_padded_data, pad_len);
         /* fill the rest with the actual signature blob */
         memcpy(blob_padded_data + pad_len, blob_orig, len);
 
@@ -1429,8 +1432,9 @@ ssh_signature pki_signature_from_blob(const ssh_key pubkey,
         case SSH_KEYTYPE_DSS:
             /* 40 is the dual signature blob len. */
             if (len != 40) {
-                ssh_pki_log("Signature has wrong size: %lu",
-                            (unsigned long)len);
+                SSH_LOG(SSH_LOG_WARN,
+                        "Signature has wrong size: %lu",
+                        (unsigned long)len);
                 ssh_signature_free(sig);
                 return NULL;
             }
@@ -1453,7 +1457,7 @@ ssh_signature pki_signature_from_blob(const ssh_key pubkey,
             }
             ssh_string_fill(r, ssh_string_data(sig_blob), 20);
 
-            pr = make_string_bn(r);
+            pr = ssh_make_string_bn(r);
             ssh_string_free(r);
             if (pr == NULL) {
                 ssh_signature_free(sig);
@@ -1467,7 +1471,7 @@ ssh_signature pki_signature_from_blob(const ssh_key pubkey,
             }
             ssh_string_fill(s, (char *)ssh_string_data(sig_blob) + 20, 20);
 
-            ps = make_string_bn(s);
+            ps = ssh_make_string_bn(s);
             ssh_string_free(s);
             if (ps == NULL) {
                 ssh_signature_free(sig);
@@ -1512,7 +1516,7 @@ ssh_signature pki_signature_from_blob(const ssh_key pubkey,
                     return NULL;
                 }
 
-                r = buffer_get_ssh_string(b);
+                r = ssh_buffer_get_ssh_string(b);
                 if (r == NULL) {
                     ssh_buffer_free(b);
                     ssh_signature_free(sig);
@@ -1523,7 +1527,7 @@ ssh_signature pki_signature_from_blob(const ssh_key pubkey,
                 ssh_print_hexa("r", ssh_string_data(r), ssh_string_len(r));
 #endif
 
-                pr = make_string_bn(r);
+                pr = ssh_make_string_bn(r);
                 ssh_string_burn(r);
                 ssh_string_free(r);
                 if (pr == NULL) {
@@ -1532,8 +1536,8 @@ ssh_signature pki_signature_from_blob(const ssh_key pubkey,
                     return NULL;
                 }
 
-                s = buffer_get_ssh_string(b);
-                rlen = buffer_get_rest_len(b);
+                s = ssh_buffer_get_ssh_string(b);
+                rlen = ssh_buffer_get_len(b);
                 ssh_buffer_free(b);
                 if (s == NULL) {
                     ssh_signature_free(sig);
@@ -1544,7 +1548,7 @@ ssh_signature pki_signature_from_blob(const ssh_key pubkey,
                 ssh_print_hexa("s", ssh_string_data(s), ssh_string_len(s));
 #endif
 
-                ps = make_string_bn(s);
+                ps = ssh_make_string_bn(s);
                 ssh_string_burn(s);
                 ssh_string_free(s);
                 if (ps == NULL) {
@@ -1559,9 +1563,10 @@ ssh_signature pki_signature_from_blob(const ssh_key pubkey,
                 }
 
                 if (rlen != 0) {
-                    ssh_pki_log("Signature has remaining bytes in inner "
-                                "sigblob: %lu",
-                                (unsigned long)rlen);
+                    SSH_LOG(SSH_LOG_WARN,
+                            "Signature has remaining bytes in inner "
+                            "sigblob: %lu",
+                            (unsigned long)rlen);
                     ssh_signature_free(sig);
                     return NULL;
                 }
@@ -1578,7 +1583,7 @@ ssh_signature pki_signature_from_blob(const ssh_key pubkey,
             break;
         default:
         case SSH_KEYTYPE_UNKNOWN:
-            ssh_pki_log("Unknown signature type");
+            SSH_LOG(SSH_LOG_WARN, "Unknown signature type");
             ssh_signature_free(sig);
             return NULL;
     }
