@@ -21,6 +21,8 @@
  * MA 02111-1307, USA.
  */
 
+#include "config.h"
+
 #include "libssh/pki.h"
 #include "libssh/pki_priv.h"
 #include "libssh/ed25519.h"
@@ -76,16 +78,18 @@ int pki_ed25519_sign(const ssh_key privkey,
     if (rc != 0) {
         goto error;
     }
-    sig->ed25519_sig = malloc(ED25519_SIG_LEN);
-    if (sig->ed25519_sig == NULL) {
-        goto error;
-    }
 
     /* This shouldn't happen */
     if (dlen - hlen != ED25519_SIG_LEN) {
         goto error;
     }
-    memcpy(sig->ed25519_sig, buffer, dlen - hlen);
+
+    sig->ed25519_sig = malloc(ED25519_SIG_LEN);
+    if (sig->ed25519_sig == NULL) {
+        goto error;
+    }
+
+    memcpy(sig->ed25519_sig, buffer, ED25519_SIG_LEN);
     SAFE_FREE(buffer);
 
     return SSH_OK;
@@ -128,8 +132,8 @@ int pki_ed25519_verify(const ssh_key pubkey,
                                   hlen + ED25519_SIG_LEN,
                                   *pubkey->ed25519_pubkey);
 
-    BURN_BUFFER(buffer, hlen + ED25519_SIG_LEN);
-    BURN_BUFFER(buffer2, hlen);
+    explicit_bzero(buffer, hlen + ED25519_SIG_LEN);
+    explicit_bzero(buffer2, hlen);
     SAFE_FREE(buffer);
     SAFE_FREE(buffer2);
     if (rc == 0) {
@@ -293,7 +297,7 @@ int pki_ed25519_sig_from_blob(ssh_signature sig, ssh_string sig_blob)
 
     len = ssh_string_len(sig_blob);
     if (len != ED25519_SIG_LEN){
-        ssh_pki_log("Invalid ssh-ed25519 signature len: %zu", len);
+        SSH_LOG(SSH_LOG_WARN, "Invalid ssh-ed25519 signature len: %zu", len);
         return SSH_ERROR;
     }
 
