@@ -31,9 +31,6 @@
 #include "libssh/crypto.h"
 #include "libssh/server.h"
 #include "libssh/socket.h"
-#ifdef WITH_SSH1
-#include "libssh/ssh1.h"
-#endif /* WITH_SSH1 */
 #include "libssh/ssh2.h"
 #include "libssh/agent.h"
 #include "libssh/packet.h"
@@ -95,7 +92,7 @@ ssh_session ssh_new(void) {
   session->maxchannel = FIRST_CHANNEL;
 
 #ifndef _WIN32
-    session->agent = agent_new(session);
+    session->agent = ssh_agent_new(session);
     if (session->agent == NULL) {
       goto err;
     }
@@ -105,14 +102,10 @@ ssh_session ssh_new(void) {
     session->opts.StrictHostKeyChecking = 1;
     session->opts.port = 0;
     session->opts.fd = -1;
-    session->opts.ssh2 = 1;
     session->opts.compressionlevel=7;
-#ifdef WITH_SSH1
-    session->opts.ssh1 = 1;
-#else
-    session->opts.ssh1 = 0;
-#endif
-
+    session->opts.nodelay = 0;
+    session->opts.flags = SSH_OPT_FLAG_PASSWORD_AUTH | SSH_OPT_FLAG_PUBKEY_AUTH |
+            SSH_OPT_FLAG_KBDINT_AUTH | SSH_OPT_FLAG_GSSAPI_AUTH;
     session->opts.identity = ssh_list_new();
     if (session->opts.identity == NULL) {
       goto err;
@@ -147,6 +140,7 @@ ssh_session ssh_new(void) {
       goto err;
     }
 
+#ifdef HAVE_DSA
     id = strdup("%d/id_dsa");
     if (id == NULL) {
       goto err;
@@ -155,15 +149,7 @@ ssh_session ssh_new(void) {
     if (rc == SSH_ERROR) {
       goto err;
     }
-
-    id = strdup("%d/identity");
-    if (id == NULL) {
-      goto err;
-    }
-    rc = ssh_list_append(session->opts.identity, id);
-    if (rc == SSH_ERROR) {
-      goto err;
-    }
+#endif
 
     return session;
 
@@ -234,7 +220,7 @@ void ssh_free(ssh_session session) {
   crypto_free(session->next_crypto);
 
 #ifndef _WIN32
-  agent_free(session->agent);
+  ssh_agent_free(session->agent);
 #endif /* _WIN32 */
 
   ssh_key_free(session->srv.dsa_key);
@@ -293,6 +279,7 @@ void ssh_free(ssh_session session) {
   SAFE_FREE(session->opts.host);
   SAFE_FREE(session->opts.sshdir);
   SAFE_FREE(session->opts.knownhosts);
+  SAFE_FREE(session->opts.global_knownhosts);
   SAFE_FREE(session->opts.ProxyCommand);
   SAFE_FREE(session->opts.gss_server_identity);
   SAFE_FREE(session->opts.gss_client_identity);
@@ -304,7 +291,7 @@ void ssh_free(ssh_session session) {
   }
 
   /* burn connection, it could contain sensitive data */
-  BURN_BUFFER(session, sizeof(struct ssh_session_struct));
+  explicit_bzero(session, sizeof(struct ssh_session_struct));
   SAFE_FREE(session);
 }
 
@@ -357,6 +344,12 @@ const char* ssh_get_kex_algo(ssh_session session) {
             return "diffie-hellman-group14-sha1";
         case SSH_KEX_ECDH_SHA2_NISTP256:
             return "ecdh-sha2-nistp256";
+        case SSH_KEX_ECDH_SHA2_NISTP384:
+            return "ecdh-sha2-nistp384";
+        case SSH_KEX_ECDH_SHA2_NISTP521:
+            return "ecdh-sha2-nistp521";
+        case SSH_KEX_CURVE25519_SHA256:
+           return "curve25519-sha256";
         case SSH_KEX_CURVE25519_SHA256_LIBSSH_ORG:
             return "curve25519-sha256@libssh.org";
         default:
@@ -799,14 +792,14 @@ const char *ssh_get_disconnect_message(ssh_session session) {
  *
  * @param session       The ssh session to use.
  *
- * @return 1 or 2, for ssh1 or ssh2, < 0 on error.
+ * @return The SSH version as integer, < 0 on error.
  */
 int ssh_get_version(ssh_session session) {
-  if (session == NULL) {
-    return -1;
-  }
+    if (session == NULL) {
+        return -1;
+    }
 
-  return session->version;
+    return 2;
 }
 
 /**
@@ -837,11 +830,7 @@ void ssh_socket_exception_callback(int code, int errno_code, void *user){
  * @return              SSH_OK on success, SSH_ERROR otherwise.
  */
 int ssh_send_ignore (ssh_session session, const char *data) {
-#ifdef WITH_SSH1
-    const int type = session->version == 1 ? SSH_MSG_IGNORE : SSH2_MSG_IGNORE;
-#else /* WITH_SSH1 */
     const int type = SSH2_MSG_IGNORE;
-#endif /* WITH_SSH1 */
     int rc;
 
     if (ssh_socket_is_open(session->socket)) {
@@ -853,7 +842,7 @@ int ssh_send_ignore (ssh_session session, const char *data) {
             ssh_set_error_oom(session);
             goto error;
         }
-        packet_send(session);
+        ssh_packet_send(session);
         ssh_handle_packets(session, 0);
     }
 
@@ -879,27 +868,17 @@ int ssh_send_debug (ssh_session session, const char *message, int always_display
     int rc;
 
     if (ssh_socket_is_open(session->socket)) {
-#ifdef WITH_SSH1
-        if (session->version == 1) {
-            rc = ssh_buffer_pack(session->out_buffer,
-                                 "bs",
-                                 SSH_MSG_DEBUG,
-                                 message);
-        } else
-#endif /* WITH_SSH1 */
-        {
-            rc = ssh_buffer_pack(session->out_buffer,
-                                 "bbsd",
-                                 SSH2_MSG_DEBUG,
-                                 always_display != 0 ? 1 : 0,
-                                 message,
-                                 0); /* empty language tag */
-        }
+        rc = ssh_buffer_pack(session->out_buffer,
+                             "bbsd",
+                             SSH2_MSG_DEBUG,
+                             always_display != 0 ? 1 : 0,
+                             message,
+                             0); /* empty language tag */
         if (rc != SSH_OK) {
             ssh_set_error_oom(session);
             goto error;
         }
-        packet_send(session);
+        ssh_packet_send(session);
         ssh_handle_packets(session, 0);
     }
 
@@ -950,5 +929,3 @@ void ssh_set_counters(ssh_session session, ssh_counter scounter,
 }
 
 /** @} */
-
-/* vim: set ts=4 sw=4 et cindent: */
