@@ -21,10 +21,10 @@
  * MA 02111-1307, USA.
  */
 
+#include "config.h"
+
 #include <errno.h>
 #include <limits.h>
-#include <stdlib.h>
-#include <string.h>
 
 #ifndef _WIN32
 #include <netinet/in.h>
@@ -129,11 +129,18 @@ struct ssh_string_struct *ssh_string_from_char(const char *what) {
  * @return The size of the content of the string, 0 on error.
  */
 size_t ssh_string_len(struct ssh_string_struct *s) {
-  if (s == NULL) {
-    return ntohl(0);
-  }
+    size_t size;
 
-  return ntohl(s->size);
+    if (s == NULL) {
+        return 0;
+    }
+
+    size = ntohl(s->size);
+    if (size > 0 && size < UINT_MAX) {
+        return size;
+    }
+
+    return 0;
 }
 
 /**
@@ -235,10 +242,11 @@ struct ssh_string_struct *ssh_string_copy(struct ssh_string_struct *s) {
  * @param[in] s         The string to burn.
  */
 void ssh_string_burn(struct ssh_string_struct *s) {
-  if (s == NULL) {
-    return;
-  }
-  memset(s->data, 'X', ssh_string_len(s));
+    if (s == NULL || s->size == 0) {
+        return;
+    }
+
+    explicit_bzero(s->data, ssh_string_len(s));
 }
 
 /**
@@ -266,5 +274,3 @@ void ssh_string_free(struct ssh_string_struct *s) {
 }
 
 /** @} */
-
-/* vim: set ts=4 sw=4 et cindent: */
