@@ -19,6 +19,8 @@
  * MA 02111-1307, USA.
  */
 
+#include "config.h"
+
 #define LIBSSH_STATIC
 
 #include "torture.h"
@@ -26,65 +28,74 @@
 #include "libssh/priv.h"
 #include "libssh/session.h"
 
+#include <sys/types.h>
+#include <pwd.h>
+
 #define BUFLEN 4096
 static char buffer[BUFLEN];
 
-static void setup(void **state) {
-    int verbosity = torture_libssh_verbosity();
-    ssh_session session = ssh_new();
+static int sshd_setup(void **state)
+{
+    torture_setup_sshd_server(state);
 
-    ssh_options_set(session, SSH_OPTIONS_HOST, "localhost");
-    ssh_options_set(session, SSH_OPTIONS_LOG_VERBOSITY, &verbosity);
-
-    *state = session;
+    return 0;
 }
 
-static void teardown(void **state) {
-    ssh_disconnect(*state);
-    ssh_free(*state);
+static int sshd_teardown(void **state) {
+    torture_teardown_sshd_server(state);
+
+    return 0;
+}
+
+static int session_setup(void **state)
+{
+    struct torture_state *s = *state;
+    struct passwd *pwd;
+
+    pwd = getpwnam("bob");
+    assert_non_null(pwd);
+    setuid(pwd->pw_uid);
+
+    s->ssh.session = torture_ssh_session(TORTURE_SSH_SERVER,
+                                         NULL,
+                                         TORTURE_SSH_USER_ALICE,
+                                         NULL);
+    assert_non_null(s->ssh.session);
+
+    return 0;
+}
+
+static int session_teardown(void **state)
+{
+    struct torture_state *s = *state;
+
+    ssh_disconnect(s->ssh.session);
+    ssh_free(s->ssh.session);
+
+    return 0;
 }
 
 static void torture_channel_read_error(void **state) {
-    ssh_session session = *state;
-    char *user = getenv("TORTURE_USER");
+    struct torture_state *s = *state;
+    ssh_session session = s->ssh.session;
     ssh_channel channel;
     int rc;
+    int fd;
     int i;
 
-    if (user == NULL) {
-        print_message("*** Please set the environment variable TORTURE_USER"
-                      " to enable this test!!\n");
-        return;
-    }
-
-    rc = ssh_options_set(session, SSH_OPTIONS_USER, user);
-    assert_true(rc == SSH_OK);
-
-    rc = ssh_connect(session);
-    assert_true(rc == SSH_OK);
-
-    rc = ssh_userauth_none(session,NULL);
-    /* This request should return a SSH_REQUEST_DENIED error */
-    if (rc == SSH_ERROR) {
-        assert_true(ssh_get_error_code(session) == SSH_REQUEST_DENIED);
-    }
-    rc = ssh_userauth_list(session, NULL);
-    assert_true(rc & SSH_AUTH_METHOD_PUBLICKEY);
-
-    rc = ssh_userauth_publickey_auto(session, NULL, NULL);
-    assert_true(rc == SSH_AUTH_SUCCESS);
-
     channel = ssh_channel_new(session);
-    assert_true(channel != NULL);
+    assert_non_null(channel);
 
     rc = ssh_channel_open_session(channel);
-    assert_true(rc == SSH_OK);
+    assert_ssh_return_code(session, rc);
 
     rc = ssh_channel_request_exec(channel, "hexdump -C /dev/urandom");
-    assert_true(rc == SSH_OK);
+    assert_ssh_return_code(session, rc);
 
     /* send crap and for server to send us a disconnect */
-    rc = write(ssh_get_fd(session),"AAAA", 4);
+    fd = ssh_get_fd(session);
+    assert_true(fd > 2);
+    rc = write(fd, "AAAA", 4);
     assert_int_equal(rc, 4);
 
     for (i=0;i<20;++i){
@@ -94,9 +105,9 @@ static void torture_channel_read_error(void **state) {
     }
 #if OPENSSH_VERSION_MAJOR == 6 && OPENSSH_VERSION_MINOR >= 7
     /* With openssh 6.7 this doesn't produce and error anymore */
-    assert_int_equal(rc, SSH_OK);
+    assert_ssh_return_code(session, rc);
 #else
-    assert_int_equal(rc, SSH_ERROR);
+    assert_ssh_return_code_equal(session, rc, SSH_ERROR);
 #endif
 
     ssh_channel_free(channel);
@@ -104,14 +115,16 @@ static void torture_channel_read_error(void **state) {
 
 int torture_run_tests(void) {
     int rc;
-    UnitTest tests[] = {
-        unit_test_setup_teardown(torture_channel_read_error, setup, teardown),
+    struct CMUnitTest tests[] = {
+        cmocka_unit_test_setup_teardown(torture_channel_read_error,
+                                        session_setup,
+                                        session_teardown),
     };
 
     ssh_init();
 
     torture_filter_tests(tests);
-    rc = run_tests(tests);
+    rc = cmocka_run_group_tests(tests, sshd_setup, sshd_teardown);
     ssh_finalize();
 
     return rc;
