@@ -1,3 +1,5 @@
+#include "config.h"
+
 #define LIBSSH_STATIC
 
 #include "torture.h"
@@ -15,9 +17,13 @@ static void torture_ssh_init(void **state) {
 }
 
 int torture_run_tests(void) {
-    const UnitTest tests[] = {
-        unit_test(torture_ssh_init),
+    int rc;
+    struct CMUnitTest tests[] = {
+        cmocka_unit_test(torture_ssh_init),
     };
 
-    return run_tests(tests);
+    torture_filter_tests(tests);
+    rc = cmocka_run_group_tests(tests, NULL, NULL);
+
+    return rc;
 }
