@@ -1,12 +1,18 @@
+#include "config.h"
+
 #define LIBSSH_STATIC
 
 #include "torture.h"
+#include "torture_key.h"
 #include "legacy.c"
 
 #define LIBSSH_RSA_TESTKEY "libssh_testkey.id_rsa"
+#ifdef HAVE_DSA
 #define LIBSSH_DSA_TESTKEY "libssh_testkey.id_dsa"
+#endif
 
-static void setup_rsa_key(void **state) {
+static int setup_rsa_key(void **state)
+{
     ssh_session session;
 
     unlink(LIBSSH_RSA_TESTKEY);
@@ -19,9 +25,13 @@ static void setup_rsa_key(void **state) {
 
     session = ssh_new();
     *state = session;
+
+    return 0;
 }
 
-static void setup_dsa_key(void **state) {
+#ifdef HAVE_DSA
+static int setup_dsa_key(void **state)
+{
     ssh_session session;
 
     unlink(LIBSSH_DSA_TESTKEY);
@@ -34,15 +44,29 @@ static void setup_dsa_key(void **state) {
 
     session = ssh_new();
     *state = session;
-}
 
-static void setup_both_keys(void **state) {
-    setup_rsa_key(state);
+    return 0;
+}
+#endif
+
+static int setup_both_keys(void **state) {
+    int rc;
+
+    rc = setup_rsa_key(state);
+    if (rc != 0) {
+        return rc;
+    }
+#ifdef HAVE_DSA
     ssh_free(*state);
-    setup_dsa_key(state);
+
+    rc = setup_dsa_key(state);
+#endif
+
+    return rc;
 }
 
-static void setup_both_keys_passphrase(void **state) {
+static int setup_both_keys_passphrase(void **state)
+{
     ssh_session session;
 
     torture_write_file(LIBSSH_RSA_TESTKEY,
@@ -50,22 +74,32 @@ static void setup_both_keys_passphrase(void **state) {
     torture_write_file(LIBSSH_RSA_TESTKEY ".pub",
                        torture_get_testkey_pub(SSH_KEYTYPE_RSA, 0));
 
+#ifdef HAVE_DSA
     torture_write_file(LIBSSH_DSA_TESTKEY,
                        torture_get_testkey(SSH_KEYTYPE_DSS, 0, 1));
     torture_write_file(LIBSSH_DSA_TESTKEY ".pub",
                        torture_get_testkey_pub(SSH_KEYTYPE_DSS, 0));
+#endif
 
     session = ssh_new();
     *state = session;
+
+    return 0;
 }
-static void teardown(void **state) {
+
+static int teardown(void **state)
+{
+#ifdef HAVE_DSA
     unlink(LIBSSH_DSA_TESTKEY);
     unlink(LIBSSH_DSA_TESTKEY ".pub");
+#endif
 
     unlink(LIBSSH_RSA_TESTKEY);
     unlink(LIBSSH_RSA_TESTKEY ".pub");
 
     ssh_free(*state);
+
+    return 0;
 }
 
 static void torture_pubkey_from_file(void **state) {
@@ -106,6 +140,7 @@ static int torture_read_one_line(const char *filename, char *buffer, size_t len)
     fclose(fp);
     return -1;
   }
+  buffer[len - 1] = '\0';
 
   fclose(fp);
 
@@ -193,12 +228,14 @@ static void torture_privatekey_from_file(void **state) {
         key = NULL;
     }
 
+#ifdef HAVE_DSA
     key = privatekey_from_file(session, LIBSSH_DSA_TESTKEY, SSH_KEYTYPE_DSS, NULL);
     assert_true(key != NULL);
     if (key != NULL) {
         privatekey_free(key);
         key = NULL;
     }
+#endif
 
     /* Test the automatic type discovery */
     key = privatekey_from_file(session, LIBSSH_RSA_TESTKEY, 0, NULL);
@@ -208,12 +245,14 @@ static void torture_privatekey_from_file(void **state) {
         key = NULL;
     }
 
+#ifdef HAVE_DSA
     key = privatekey_from_file(session, LIBSSH_DSA_TESTKEY, 0, NULL);
     assert_true(key != NULL);
     if (key != NULL) {
         privatekey_free(key);
         key = NULL;
     }
+#endif
 }
 
 /**
@@ -230,12 +269,14 @@ static void torture_privatekey_from_file_passphrase(void **state) {
         key = NULL;
     }
 
+#ifdef HAVE_DSA
     key = privatekey_from_file(session, LIBSSH_DSA_TESTKEY, SSH_KEYTYPE_DSS, TORTURE_TESTKEY_PASSWORD);
     assert_true(key != NULL);
     if (key != NULL) {
         privatekey_free(key);
         key = NULL;
     }
+#endif
 
     /* Test the automatic type discovery */
     key = privatekey_from_file(session, LIBSSH_RSA_TESTKEY, 0, TORTURE_TESTKEY_PASSWORD);
@@ -245,33 +286,37 @@ static void torture_privatekey_from_file_passphrase(void **state) {
         key = NULL;
     }
 
+#ifdef HAVE_DSA
     key = privatekey_from_file(session, LIBSSH_DSA_TESTKEY, 0, TORTURE_TESTKEY_PASSWORD);
     assert_true(key != NULL);
     if (key != NULL) {
         privatekey_free(key);
         key = NULL;
     }
+#endif
 }
 
 int torture_run_tests(void) {
     int rc;
-    UnitTest tests[] = {
-        unit_test_setup_teardown(torture_pubkey_from_file,
-                                 setup_rsa_key,
-                                 teardown),
-        unit_test_setup_teardown(torture_pubkey_generate_from_privkey,
-                                 setup_rsa_key, teardown),
-        unit_test_setup_teardown(torture_privatekey_from_file,
-                                 setup_both_keys,
-                                 teardown),
-        unit_test_setup_teardown(torture_privatekey_from_file_passphrase,
-                                 setup_both_keys_passphrase, teardown),
+    struct CMUnitTest tests[] = {
+        cmocka_unit_test_setup_teardown(torture_pubkey_from_file,
+                                        setup_rsa_key,
+                                        teardown),
+        cmocka_unit_test_setup_teardown(torture_pubkey_generate_from_privkey,
+                                        setup_rsa_key,
+                                        teardown),
+        cmocka_unit_test_setup_teardown(torture_privatekey_from_file,
+                                        setup_both_keys,
+                                        teardown),
+        cmocka_unit_test_setup_teardown(torture_privatekey_from_file_passphrase,
+                                        setup_both_keys_passphrase,
+                                        teardown),
     };
 
 
     ssh_init();
     torture_filter_tests(tests);
-    rc=run_tests(tests);
+    rc = cmocka_run_group_tests(tests, NULL, NULL);
     ssh_finalize();
     return rc;
 }
